# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

from azure.core.pipeline.transport import AsyncHttpTransport


class AsyncTransportWrapper(AsyncHttpTransport):
    """Wrapper class that ensures that an inner client created
    by a `get_client` method does not close the outer transport for the parent
    when used in a context manager.
    """

    def __init__(self, async_transport):
        self._transport = async_transport

    async def send(self, request, **kwargs):
        return await self._transport.send(request, **kwargs)

    async def open(self):
        pass

    async def close(self):
        pass

    async def __aenter__(self):
        pass

    async def __aexit__(self, *args):  # pylint: disable=arguments-differ
        pass



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

# pylint: disable=protected-access

import uuid
from typing import (
    Any,
    Union,
    List,
    overload,
    Optional,
    Mapping
)
from azure.core.credentials import AzureKeyCredential
from azure.core.credentials_async import AsyncTokenCredential
from azure.core.polling.async_base_polling import AsyncLROBasePolling
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.tracing.decorator import distributed_trace
from azure.core.pipeline import AsyncPipeline
from azure.core.async_paging import AsyncItemPaged
from ._helpers_async import AsyncTransportWrapper
from ._document_analysis_client_async import DocumentAnalysisClient
from ._async_polling import AsyncDocumentModelAdministrationLROPoller
from ._form_base_client_async import FormRecognizerClientBaseAsync
from .._api_versions import DocumentAnalysisApiVersion
from .._polling import DocumentModelAdministrationPolling
from .._models import (
    ModelBuildMode,
    ClassifierDocumentTypeDetails,
    DocumentClassifierDetails,
    DocumentModelDetails,
    DocumentModelSummary,
    OperationDetails,
    OperationSummary,
    ResourceDetails,
    TargetAuthorization,
)


class DocumentModelAdministrationClient(FormRecognizerClientBaseAsync):
    """DocumentModelAdministrationClient is the Form Recognizer interface to use for building
    and managing models.

    It provides methods for building models and classifiers, as well as methods for viewing and
    deleting models and classifiers, viewing model and classifier operations, accessing account
    information, copying models to another Form Recognizer resource, and composing a new model
    from a collection of existing models.

    .. note:: DocumentModelAdministrationClient should be used with API versions
        2022-08-31 and up. To use API versions <=v2.1, instantiate a FormTrainingClient.

    :param str endpoint: Supported Cognitive Services endpoints (protocol and hostname,
        for example: https://westus2.api.cognitive.microsoft.com).
    :param credential: Credentials needed for the client to connect to Azure.
        This is an instance of AzureKeyCredential if using an API key or a token
        credential from :mod:`azure.identity`.
    :type credential: :class:`~azure.core.credentials.AzureKeyCredential` or
        :class:`~azure.core.credentials.TokenCredential`
    :keyword api_version:
        The API version of the service to use for requests. It defaults to the latest service version.
        Setting to an older version may result in reduced feature compatibility. To use API versions
        <=v2.1, instantiate a FormTrainingClient.
    :paramtype api_version: str or ~azure.ai.formrecognizer.DocumentAnalysisApiVersion

    .. versionadded:: 2022-08-31
        The *DocumentModelAdministrationClient* and its client methods.

    .. admonition:: Example:

        .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_authentication_async.py
            :start-after: [START create_dt_client_with_key_async]
            :end-before: [END create_dt_client_with_key_async]
            :language: python
            :dedent: 4
            :caption: Creating the DocumentModelAdministrationClient with an endpoint and API key.

        .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_authentication_async.py
            :start-after: [START create_dt_client_with_aad_async]
            :end-before: [END create_dt_client_with_aad_async]
            :language: python
            :dedent: 4
            :caption: Creating the DocumentModelAdministrationClient with a token credential.
    """

    def __init__(
        self, endpoint: str, credential: Union[AzureKeyCredential, AsyncTokenCredential], **kwargs: Any
    ) -> None:
        api_version = kwargs.pop("api_version", DocumentAnalysisApiVersion.V2023_07_31)
        super().__init__(
            endpoint=endpoint, credential=credential, api_version=api_version, client_kind="document", **kwargs
        )

    @overload
    async def begin_build_document_model(
        self,
        build_mode: Union[str, ModelBuildMode],
        *,
        blob_container_url: str,
        prefix: Optional[str] = None,
        model_id: Optional[str] = None,
        description: Optional[str] = None,
        tags: Optional[Mapping[str, str]] = None,
        **kwargs: Any
    ) -> AsyncDocumentModelAdministrationLROPoller[DocumentModelDetails]:
        """Build a custom document model.

        The request must include a `blob_container_url` keyword parameter that is an
        externally accessible Azure storage blob container URI (preferably a Shared Access Signature URI). Note that
        a container URI (without SAS) is accepted only when the container is public or has a managed identity
        configured, see more about configuring managed identities to work with Form Recognizer here:
        https://docs.microsoft.com/azure/applied-ai-services/form-recognizer/managed-identities.
        Models are built using documents that are of the following content type - 'application/pdf',
        'image/jpeg', 'image/png', 'image/tiff', 'image/bmp', or 'image/heif'. Other types of content in the container
        is ignored.

        :param build_mode: The custom model build mode. Possible values include: "template", "neural".
            For more information about build modes, see: https://aka.ms/azsdk/formrecognizer/buildmode.
        :type build_mode: str or :class:`~azure.ai.formrecognizer.ModelBuildMode`
        :keyword str blob_container_url: An Azure Storage blob container's SAS URI. A container URI (without SAS)
            can be used if the container is public or has a managed identity configured. For more information on
            setting up a training data set, see: https://aka.ms/azsdk/formrecognizer/buildtrainingset.
        :keyword str model_id: A unique ID for your model. If not specified, a model ID will be created for you.
        :keyword str description: An optional description to add to the model.
        :keyword str prefix: A case-sensitive prefix string to filter documents in the blob container url path.
            For example, when using an Azure storage blob URI, use the prefix to restrict sub folders.
            `prefix` should end in '/' to avoid cases where filenames share the same prefix.
        :keyword tags: List of user defined key-value tag attributes associated with the model.
        :paramtype tags: dict[str, str]
        :return: An instance of an AsyncDocumentModelAdministrationLROPoller. Call `result()` on the poller
            object to return a :class:`~azure.ai.formrecognizer.DocumentModelDetails`.
        :rtype: ~azure.ai.formrecognizer.aio.AsyncDocumentModelAdministrationLROPoller[DocumentModelDetails]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def begin_build_document_model(
        self,
        build_mode: Union[str, ModelBuildMode],
        *,
        blob_container_url: str,
        file_list: str,
        model_id: Optional[str] = None,
        description: Optional[str] = None,
        tags: Optional[Mapping[str, str]] = None,
        **kwargs: Any
    ) -> AsyncDocumentModelAdministrationLROPoller[DocumentModelDetails]:
        """Build a custom document model.

        The request must include a `blob_container_url` keyword parameter that is an
        externally accessible Azure storage blob container URI (preferably a Shared Access Signature URI). Note that
        a container URI (without SAS) is accepted only when the container is public or has a managed identity
        configured, see more about configuring managed identities to work with Form Recognizer here:
        https://docs.microsoft.com/azure/applied-ai-services/form-recognizer/managed-identities.
        Models are built using documents that are of the following content type - 'application/pdf',
        'image/jpeg', 'image/png', 'image/tiff', 'image/bmp', or 'image/heif'. Other types of content in the container
        is ignored.

        :param build_mode: The custom model build mode. Possible values include: "template", "neural".
            For more information about build modes, see: https://aka.ms/azsdk/formrecognizer/buildmode.
        :type build_mode: str or :class:`~azure.ai.formrecognizer.ModelBuildMode`
        :keyword str blob_container_url: An Azure Storage blob container's SAS URI. A container URI (without SAS)
            can be used if the container is public or has a managed identity configured. For more information on
            setting up a training data set, see: https://aka.ms/azsdk/formrecognizer/buildtrainingset.
        :keyword str model_id: A unique ID for your model. If not specified, a model ID will be created for you.
        :keyword str description: An optional description to add to the model.
        :keyword str file_list: Path to a JSONL file within the container specifying a subset of
            documents for training.
        :keyword tags: List of user defined key-value tag attributes associated with the model.
        :paramtype tags: dict[str, str]
        :return: An instance of an AsyncDocumentModelAdministrationLROPoller. Call `result()` on the poller
            object to return a :class:`~azure.ai.formrecognizer.DocumentModelDetails`.
        :rtype: ~azure.ai.formrecognizer.aio.AsyncDocumentModelAdministrationLROPoller[DocumentModelDetails]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def begin_build_document_model(
        self, build_mode: Union[str, ModelBuildMode], *, blob_container_url: str, **kwargs: Any
    ) -> AsyncDocumentModelAdministrationLROPoller[DocumentModelDetails]:
        """Build a custom document model.

        The request must include a `blob_container_url` keyword parameter that is an
        externally accessible Azure storage blob container URI (preferably a Shared Access Signature URI). Note that
        a container URI (without SAS) is accepted only when the container is public or has a managed identity
        configured, see more about configuring managed identities to work with Form Recognizer here:
        https://docs.microsoft.com/azure/applied-ai-services/form-recognizer/managed-identities.
        Models are built using documents that are of the following content type - 'application/pdf',
        'image/jpeg', 'image/png', 'image/tiff', 'image/bmp', or 'image/heif'. Other types of content in the container
        is ignored.

        :param build_mode: The custom model build mode. Possible values include: "template", "neural".
            For more information about build modes, see: https://aka.ms/azsdk/formrecognizer/buildmode.
        :type build_mode: str or :class:`~azure.ai.formrecognizer.ModelBuildMode`
        :keyword str blob_container_url: An Azure Storage blob container's SAS URI. A container URI (without SAS)
            can be used if the container is public or has a managed identity configured. For more information on
            setting up a training data set, see: https://aka.ms/azsdk/formrecognizer/buildtrainingset.
        :keyword str model_id: A unique ID for your model. If not specified, a model ID will be created for you.
        :keyword str description: An optional description to add to the model.
        :keyword str prefix: A case-sensitive prefix string to filter documents in the blob container url path.
            For example, when using an Azure storage blob URI, use the prefix to restrict sub folders.
            `prefix` should end in '/' to avoid cases where filenames share the same prefix.
        :keyword str file_list: Path to a JSONL file within the container specifying a subset of
            documents for training.
        :keyword tags: List of user defined key-value tag attributes associated with the model.
        :paramtype tags: dict[str, str]
        :return: An instance of an AsyncDocumentModelAdministrationLROPoller. Call `result()` on the poller
            object to return a :class:`~azure.ai.formrecognizer.DocumentModelDetails`.
        :rtype: ~azure.ai.formrecognizer.aio.AsyncDocumentModelAdministrationLROPoller[DocumentModelDetails]
        :raises ~azure.core.exceptions.HttpResponseError:

        .. versionadded:: 2023-07-31
            The *file_list* keyword argument.

        .. admonition:: Example:

            .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_build_model_async.py
                :start-after: [START build_model_async]
                :end-before: [END build_model_async]
                :language: python
                :dedent: 4
                :caption: Building a model from training files.
        """

        def callback(raw_response, _, headers):  # pylint: disable=unused-argument
            op_response = self._deserialize(self._generated_models.DocumentModelBuildOperationDetails, raw_response)
            model_info = self._deserialize(self._generated_models.DocumentModelDetails, op_response.result)
            return DocumentModelDetails._from_generated(model_info)

        description = kwargs.pop("description", None)
        model_id = kwargs.pop("model_id", None)
        tags = kwargs.pop("tags", None)
        cls = kwargs.pop("cls", callback)
        continuation_token = kwargs.pop("continuation_token", None)
        polling_interval = kwargs.pop("polling_interval", self._client._config.polling_interval)
        prefix = kwargs.pop("prefix", None)
        file_list = kwargs.pop("file_list", None)

        if model_id is None:
            model_id = str(uuid.uuid4())

        azure_blob_source = None
        azure_blob_file_list_source = None
        if prefix:
            azure_blob_source = self._generated_models.AzureBlobContentSource(
                container_url=blob_container_url,
                prefix=prefix
            )
        if file_list:
            azure_blob_file_list_source = self._generated_models.AzureBlobFileListContentSource(
                container_url=blob_container_url,
                file_list=file_list
            )
        if not azure_blob_source and not azure_blob_file_list_source:
            azure_blob_source = self._generated_models.AzureBlobContentSource(
                container_url=blob_container_url,
            )

        model_kwargs = {}
        if self._api_version == DocumentAnalysisApiVersion.V2022_08_31:
            _client_op_path = self._client.begin_build_document_model
            if file_list:
                raise ValueError(
                    "Keyword argument 'file_list' is only available for API version V2023_07_31 and later."
                )
        else:
            _client_op_path = self._client.document_models.begin_build_model
            model_kwargs.update({"azure_blob_file_list_source": azure_blob_file_list_source})
        return await _client_op_path(  # type: ignore
            build_request=self._generated_models.BuildDocumentModelRequest(
                model_id=model_id,
                build_mode=build_mode,
                description=description,
                tags=tags,
                azure_blob_source=azure_blob_source,
                **model_kwargs
            ),
            cls=cls,
            continuation_token=continuation_token,
            polling=AsyncLROBasePolling(
                timeout=polling_interval, lro_algorithms=[DocumentModelAdministrationPolling()], **kwargs
            ),
            **kwargs
        )

    @distributed_trace_async
    async def begin_compose_document_model(
        self, component_model_ids: List[str], **kwargs: Any
    ) -> AsyncDocumentModelAdministrationLROPoller[DocumentModelDetails]:
        """Creates a composed document model from a collection of existing models.

        A composed model allows multiple models to be called with a single model ID. When a document is
        submitted to be analyzed with a composed model ID, a classification step is first performed to
        route it to the correct custom model.

        :param list[str] component_model_ids: List of model IDs to use in the composed model.
        :keyword str model_id: A unique ID for your composed model.
            If not specified, a model ID will be created for you.
        :keyword str description: An optional description to add to the model.
        :keyword tags: List of user defined key-value tag attributes associated with the model.
        :paramtype tags: dict[str, str]
        :return: An instance of an AsyncDocumentModelAdministrationLROPoller. Call `result()` on the poller
            object to return a :class:`~azure.ai.formrecognizer.DocumentModelDetails`.
        :rtype: ~azure.ai.formrecognizer.aio.AsyncDocumentModelAdministrationLROPoller[DocumentModelDetails]
        :raises ~azure.core.exceptions.HttpResponseError:

        .. admonition:: Example:

            .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_compose_model_async.py
                :start-after: [START composed_model_async]
                :end-before: [END composed_model_async]
                :language: python
                :dedent: 4
                :caption: Creating a composed model with existing models.
        """

        def _compose_callback(raw_response, _, headers):  # pylint: disable=unused-argument
            op_response = self._deserialize(self._generated_models.DocumentModelComposeOperationDetails, raw_response)
            model_info = self._deserialize(self._generated_models.DocumentModelDetails, op_response.result)
            return DocumentModelDetails._from_generated(model_info)

        model_id = kwargs.pop("model_id", None)
        description = kwargs.pop("description", None)
        tags = kwargs.pop("tags", None)
        continuation_token = kwargs.pop("continuation_token", None)
        polling_interval = kwargs.pop("polling_interval", self._client._config.polling_interval)

        if model_id is None:
            model_id = str(uuid.uuid4())

        if self._api_version == DocumentAnalysisApiVersion.V2022_08_31:
            _client_op_path = self._client.begin_compose_document_model
        else:
            _client_op_path = self._client.document_models.begin_compose_model
        return await _client_op_path(  # type: ignore
            compose_request=self._generated_models.ComposeDocumentModelRequest(
                model_id=model_id,
                description=description,
                tags=tags,
                component_models=[
                    self._generated_models.ComponentDocumentModelDetails(model_id=model_id)
                    for model_id in component_model_ids
                ]
                if component_model_ids
                else [],
            ),
            cls=kwargs.pop("cls", _compose_callback),
            polling=AsyncLROBasePolling(
                timeout=polling_interval, lro_algorithms=[DocumentModelAdministrationPolling()], **kwargs
            ),
            continuation_token=continuation_token,
            **kwargs
        )

    @distributed_trace_async
    async def get_copy_authorization(self, **kwargs: Any) -> TargetAuthorization:
        """Generate authorization for copying a custom model into the target Form Recognizer resource.

        This should be called by the target resource (where the model will be copied to)
        and the output can be passed as the `target` parameter into :func:`~begin_copy_document_model_to()`.

        :keyword str model_id: A unique ID for your copied model.
            If not specified, a model ID will be created for you.
        :keyword str description: An optional description to add to the model.
        :keyword tags: List of user defined key-value tag attributes associated with the model.
        :paramtype tags: dict[str, str]
        :return: A dictionary with values necessary for the copy authorization.
        :rtype: ~azure.ai.formrecognizer.TargetAuthorization
        :raises ~azure.core.exceptions.HttpResponseError:
        """

        model_id = kwargs.pop("model_id", None)
        description = kwargs.pop("description", None)
        tags = kwargs.pop("tags", None)

        if model_id is None:
            model_id = str(uuid.uuid4())

        if self._api_version == DocumentAnalysisApiVersion.V2022_08_31:
            _client_op_path = self._client.authorize_copy_document_model
        else:
            _client_op_path = self._client.document_models.authorize_model_copy
        response = await _client_op_path(
            authorize_copy_request=self._generated_models.AuthorizeCopyRequest(
                model_id=model_id, description=description, tags=tags
            ),
            **kwargs
        )
        target = response.serialize()  # type: ignore
        return target

    @distributed_trace_async
    async def begin_copy_document_model_to(
        self, model_id: str, target: TargetAuthorization, **kwargs: Any
    ) -> AsyncDocumentModelAdministrationLROPoller[DocumentModelDetails]:
        """Copy a document model stored in this resource (the source) to the user specified
        target Form Recognizer resource.

        This should be called with the source Form Recognizer resource
        (with the model that is intended to be copied). The `target` parameter should be supplied from the
        target resource's output from calling the :func:`~get_copy_authorization()` method.

        :param str model_id: Model identifier of the model to copy to target resource.
        :param ~azure.ai.formrecognizer.TargetAuthorization target:
            The copy authorization generated from the target resource's call to
            :func:`~get_copy_authorization()`.
        :return: An instance of a AsyncDocumentModelAdministrationLROPoller. Call `result()` on the poller
            object to return a :class:`~azure.ai.formrecognizer.DocumentModelDetails`.
        :rtype: ~azure.ai.formrecognizer.aio.AsyncDocumentModelAdministrationLROPoller[DocumentModelDetails]
        :raises ~azure.core.exceptions.HttpResponseError:

        .. admonition:: Example:

            .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_copy_model_to_async.py
                :start-after: [START begin_copy_document_model_to_async]
                :end-before: [END begin_copy_document_model_to_async]
                :language: python
                :dedent: 4
                :caption: Copy a model from the source resource to the target resource
        """

        def _copy_callback(raw_response, _, headers):  # pylint: disable=unused-argument
            op_response = self._deserialize(self._generated_models.DocumentModelCopyToOperationDetails, raw_response)
            model_info = self._deserialize(self._generated_models.DocumentModelDetails, op_response.result)
            return DocumentModelDetails._from_generated(model_info)

        if not model_id:
            raise ValueError("model_id cannot be None or empty.")

        polling_interval = kwargs.pop("polling_interval", self._client._config.polling_interval)
        continuation_token = kwargs.pop("continuation_token", None)

        if self._api_version == DocumentAnalysisApiVersion.V2022_08_31:
            _client_op_path = self._client.begin_copy_document_model_to
        else:
            _client_op_path = self._client.document_models.begin_copy_model_to
        return await _client_op_path(  # type: ignore
            model_id=model_id,
            copy_to_request=self._generated_models.CopyAuthorization(
                target_resource_id=target["targetResourceId"],
                target_resource_region=target["targetResourceRegion"],
                target_model_id=target["targetModelId"],
                access_token=target["accessToken"],
                expiration_date_time=target["expirationDateTime"],
                target_model_location=target["targetModelLocation"],
            )
            if target
            else None,
            cls=kwargs.pop("cls", _copy_callback),
            polling=AsyncLROBasePolling(
                timeout=polling_interval, lro_algorithms=[DocumentModelAdministrationPolling()], **kwargs
            ),
            continuation_token=continuation_token,
            **kwargs
        )

    @distributed_trace_async
    async def delete_document_model(self, model_id: str, **kwargs: Any) -> None:
        """Delete a custom document model.

        :param model_id: Model identifier.
        :type model_id: str
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError or ~azure.core.exceptions.ResourceNotFoundError:

        .. admonition:: Example:

            .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_manage_models_async.py
                :start-after: [START delete_document_model_async]
                :end-before: [END delete_document_model_async]
                :language: python
                :dedent: 8
                :caption: Delete a model.
        """

        if not model_id:
            raise ValueError("model_id cannot be None or empty.")

        if self._api_version == DocumentAnalysisApiVersion.V2022_08_31:
            _client_op_path = self._client.delete_document_model
        else:
            _client_op_path = self._client.document_models.delete_model
        return await _client_op_path(model_id=model_id, **kwargs)

    @distributed_trace
    def list_document_models(self, **kwargs: Any) -> AsyncItemPaged[DocumentModelSummary]:
        """List information for each model, including its model ID,
        description, and when it was created.

        :return: Pageable of DocumentModelSummary.
        :rtype: ~azure.core.async_paging.AsyncItemPaged[DocumentModelSummary]
        :raises ~azure.core.exceptions.HttpResponseError:

        .. admonition:: Example:

            .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_manage_models_async.py
                :start-after: [START list_document_models_async]
                :end-before: [END list_document_models_async]
                :language: python
                :dedent: 8
                :caption: List all models that were built successfully under the Form Recognizer resource.
        """

        if self._api_version == DocumentAnalysisApiVersion.V2022_08_31:
            _client_op_path = self._client.get_document_models
        else:
            _client_op_path = self._client.document_models.list_models
        return _client_op_path(  # type: ignore
            cls=kwargs.pop(
                "cls",
                lambda objs: [DocumentModelSummary._from_generated(x) for x in objs],
            ),
            **kwargs
        )

    @distributed_trace_async
    async def get_resource_details(self, **kwargs: Any) -> ResourceDetails:
        """Get information about the models under the Form Recognizer resource.

        :return: Summary of custom models under the resource - model count and limit.
        :rtype: ~azure.ai.formrecognizer.ResourceDetails
        :raises ~azure.core.exceptions.HttpResponseError:

        .. admonition:: Example:

            .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_manage_models_async.py
                :start-after: [START get_resource_details_async]
                :end-before: [END get_resource_details_async]
                :language: python
                :dedent: 4
                :caption: Get model counts and limits under the Form Recognizer resource.
        """

        if self._api_version == DocumentAnalysisApiVersion.V2022_08_31:
            _client_op_path = self._client.get_resource_details
        else:
            _client_op_path = self._client.miscellaneous.get_resource_info
        response = await _client_op_path(**kwargs)
        return ResourceDetails._from_generated(response)

    @distributed_trace_async
    async def get_document_model(self, model_id: str, **kwargs: Any) -> DocumentModelDetails:
        """Get a document model by its ID.

        :param str model_id: Model identifier.
        :return: DocumentModelDetails
        :rtype: ~azure.ai.formrecognizer.DocumentModelDetails
        :raises ~azure.core.exceptions.HttpResponseError or ~azure.core.exceptions.ResourceNotFoundError:

        .. admonition:: Example:

            .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_manage_models_async.py
                :start-after: [START get_document_model_async]
                :end-before: [END get_document_model_async]
                :language: python
                :dedent: 8
                :caption: Get a model by its ID.
        """

        if not model_id:
            raise ValueError("model_id cannot be None or empty.")

        if self._api_version == DocumentAnalysisApiVersion.V2022_08_31:
            _client_op_path = self._client.get_document_model
        else:
            _client_op_path = self._client.document_models.get_model
        response = await _client_op_path(model_id=model_id, **kwargs)
        return DocumentModelDetails._from_generated(response)

    @distributed_trace
    def list_operations(self, **kwargs: Any) -> AsyncItemPaged[OperationSummary]:
        """List information for each operation.

        Lists all operations associated with the Form Recognizer resource.
        Note that operation information only persists for 24 hours. If the document model operation was successful,
        the document model can be accessed using the :func:`~get_document_model` or :func:`~list_document_models` APIs.

        :return: A pageable of OperationSummary.
        :rtype: ~azure.core.async_paging.AsyncItemPaged[OperationSummary]
        :raises ~azure.core.exceptions.HttpResponseError:

        .. admonition:: Example:

            .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_get_operations_async.py
                :start-after: [START list_operations_async]
                :end-before: [END list_operations_async]
                :language: python
                :dedent: 4
                :caption: List all document model operations in the past 24 hours.
        """

        if self._api_version == DocumentAnalysisApiVersion.V2022_08_31:
            _client_op_path = self._client.get_operations
        else:
            _client_op_path = self._client.miscellaneous.list_operations
        return _client_op_path(  # type: ignore
            cls=kwargs.pop(
                "cls",
                lambda objs: [OperationSummary._from_generated(x) for x in objs],
            ),
            **kwargs
        )

    @distributed_trace_async
    async def get_operation(self, operation_id: str, **kwargs: Any) -> OperationDetails:
        """Get an operation by its ID.

        Get an operation associated with the Form Recognizer resource.
        Note that operation information only persists for 24 hours. If the document model operation was successful,
        the model can be accessed using the :func:`~get_document_model` or :func:`~list_document_models` APIs.

        :param str operation_id: The operation ID.
        :return: OperationDetails
        :rtype: ~azure.ai.formrecognizer.OperationDetails
        :raises ~azure.core.exceptions.HttpResponseError:

        .. admonition:: Example:

            .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_get_operations_async.py
                :start-after: [START get_operation_async]
                :end-before: [END get_operation_async]
                :language: python
                :dedent: 8
                :caption: Get a document model operation by its ID.
        """

        if not operation_id:
            raise ValueError("'operation_id' cannot be None or empty.")

        if self._api_version == DocumentAnalysisApiVersion.V2022_08_31:
            _client_op_path = self._client.get_operation
        else:
            _client_op_path = self._client.miscellaneous.get_operation
        return OperationDetails._from_generated(
            await _client_op_path(operation_id, **kwargs),
            api_version=self._api_version,
        )

    @distributed_trace_async
    async def begin_build_document_classifier(
        self,
        doc_types: Mapping[str, ClassifierDocumentTypeDetails],
        *,
        classifier_id: Optional[str] = None,
        description: Optional[str] = None,
        **kwargs: Any
    ) -> AsyncDocumentModelAdministrationLROPoller[DocumentClassifierDetails]:
        """Build a document classifier. For more information on how to build and train
        a custom classifier model, see https://aka.ms/azsdk/formrecognizer/buildclassifiermodel.

        :param doc_types: Mapping of document types to classify against.
        :type doc_types: Mapping[str, ~azure.ai.formrecognizer.ClassifierDocumentTypeDetails]
        :keyword str classifier_id: Unique document classifier name.
            If not specified, a classifier ID will be created for you.
        :keyword str description: Document classifier description.
        :return: An instance of an AsyncDocumentModelAdministrationLROPoller. Call `result()` on the poller
            object to return a :class:`~azure.ai.formrecognizer.DocumentClassifierDetails`.
        :rtype: ~azure.ai.formrecognizer.aio.AsyncDocumentModelAdministrationLROPoller[DocumentClassifierDetails]
        :raises ~azure.core.exceptions.HttpResponseError:

        .. versionadded:: 2023-07-31
            The *begin_build_document_classifier* client method.

        .. admonition:: Example:

            .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_build_classifier_async.py
                :start-after: [START build_classifier_async]
                :end-before: [END build_classifier_async]
                :language: python
                :dedent: 4
                :caption: Build a document classifier.
        """
        def callback(raw_response, _, headers):  # pylint: disable=unused-argument
            op_response = \
                self._deserialize(self._generated_models.DocumentClassifierBuildOperationDetails, raw_response)
            model_info = self._deserialize(self._generated_models.DocumentClassifierDetails, op_response.result)
            return DocumentClassifierDetails._from_generated(model_info)

        if self._api_version == DocumentAnalysisApiVersion.V2022_08_31:
            raise ValueError("Method 'begin_build_document_classifier()' is only available for API version "
                             "V2023_07_31 and later")
        cls = kwargs.pop("cls", callback)
        continuation_token = kwargs.pop("continuation_token", None)
        polling_interval = kwargs.pop("polling_interval", self._client._config.polling_interval)
        if classifier_id is None:
            classifier_id = str(uuid.uuid4())

        _doc_types = {doc: details._to_generated() for doc, details in doc_types.items()}

        return await self._client.document_classifiers.begin_build_classifier(
            build_request=self._generated_models.BuildDocumentClassifierRequest(
                classifier_id=classifier_id,
                description=description,
                doc_types=_doc_types,
            ),
            cls=cls,
            continuation_token=continuation_token,
            polling=AsyncLROBasePolling(
                timeout=polling_interval, lro_algorithms=[DocumentModelAdministrationPolling()], **kwargs
            ),
            **kwargs
        )

    @distributed_trace_async
    async def get_document_classifier(self, classifier_id: str, **kwargs: Any) -> DocumentClassifierDetails:
        """Get a document classifier by its ID.

        :param str classifier_id: Classifier identifier.
        :return: DocumentClassifierDetails
        :rtype: ~azure.ai.formrecognizer.DocumentClassifierDetails
        :raises ~azure.core.exceptions.HttpResponseError or ~azure.core.exceptions.ResourceNotFoundError:

        .. versionadded:: 2023-07-31
            The *get_document_classifier* client method.

        .. admonition:: Example:

            .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_manage_classifiers_async.py
                :start-after: [START get_document_classifier_async]
                :end-before: [END get_document_classifier_async]
                :language: python
                :dedent: 8
                :caption: Get a classifier by its ID.
        """

        if not classifier_id:
            raise ValueError("classifier_id cannot be None or empty.")

        if self._api_version == DocumentAnalysisApiVersion.V2022_08_31:
            raise ValueError("Method 'get_document_classifier()' is only available for API version "
                             "V2023_07_31 and later")
        response = await self._client.document_classifiers.get_classifier(classifier_id=classifier_id, **kwargs)
        return DocumentClassifierDetails._from_generated(response)

    @distributed_trace
    def list_document_classifiers(self, **kwargs: Any) -> AsyncItemPaged[DocumentClassifierDetails]:
        """List information for each document classifier, including its classifier ID,
        description, and when it was created.

        :return: Pageable of DocumentClassifierDetails.
        :rtype: ~azure.core.async_paging.AsyncItemPaged[DocumentClassifierDetails]
        :raises ~azure.core.exceptions.HttpResponseError:

        .. versionadded:: 2023-07-31
            The *list_document_classifiers* client method.

        .. admonition:: Example:

            .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_manage_classifiers_async.py
                :start-after: [START list_document_classifiers_async]
                :end-before: [END list_document_classifiers_async]
                :language: python
                :dedent: 8
                :caption: List all classifiers that were built successfully under the Form Recognizer resource.
        """

        if self._api_version == DocumentAnalysisApiVersion.V2022_08_31:
            raise ValueError("Method 'list_document_classifiers()' is only available for API version "
                             "V2023_07_31 and later")
        return self._client.document_classifiers.list_classifiers(  # type: ignore
            cls=kwargs.pop(
                "cls",
                lambda objs: [DocumentClassifierDetails._from_generated(x) for x in objs],
            ),
            **kwargs
        )

    @distributed_trace_async
    async def delete_document_classifier(self, classifier_id: str, **kwargs: Any) -> None:
        """Delete a document classifier.

        :param str classifier_id: Classifier identifier.
        :return: None
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError or ~azure.core.exceptions.ResourceNotFoundError:

        .. versionadded:: 2023-07-31
            The *delete_document_classifier* client method.

        .. admonition:: Example:

            .. literalinclude:: ../samples/v3.2_and_later/async_samples/sample_manage_classifiers_async.py
                :start-after: [START delete_document_classifier_async]
                :end-before: [END delete_document_classifier_async]
                :language: python
                :dedent: 8
                :caption: Delete a classifier.
        """

        if not classifier_id:
            raise ValueError("classifier_id cannot be None or empty.")

        if self._api_version == DocumentAnalysisApiVersion.V2022_08_31:
            raise ValueError("Method 'delete_document_classifier()' is only available for API version "
                             "V2023_07_31 and later")
        return await self._client.document_classifiers.delete_classifier(classifier_id=classifier_id, **kwargs)

    def get_document_analysis_client(self, **kwargs: Any) -> DocumentAnalysisClient:
        """Get an instance of a DocumentAnalysisClient from DocumentModelAdministrationClient.

        :rtype: ~azure.ai.formrecognizer.aio.DocumentAnalysisClient
        :return: A DocumentAnalysisClient
        """

        _pipeline = AsyncPipeline(
            transport=AsyncTransportWrapper(self._client._client._pipeline._transport),
            policies=self._client._client._pipeline._impl_policies,  # type: ignore
        )
        client = DocumentAnalysisClient(
            endpoint=self._endpoint,
            credential=self._credential,
            pipeline=_pipeline,
            api_version=self._api_version,
            **kwargs
        )
        # need to share config, but can't pass as a keyword into client
        client._client._config = self._client._config
        return client

    async def __aenter__(self) -> "DocumentModelAdministrationClient":
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *args: "Any") -> None:
        await self._client.__aexit__(*args)

    async def close(self) -> None:
        """Close the :class:`~azure.ai.formrecognizer.aio.DocumentModelAdministrationClient` session."""
        await self._client.__aexit__()



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

import functools
from devtools_testutils import recorded_by_proxy, set_bodiless_matcher
from azure.core.rest import HttpRequest
from azure.ai.formrecognizer import DocumentModelAdministrationClient, FormTrainingClient
from testcase import FormRecognizerTest
from preparers import GlobalClientPreparer as _GlobalClientPreparer
from preparers import FormRecognizerPreparer

DocumentModelAdministrationClientPreparer = functools.partial(_GlobalClientPreparer, DocumentModelAdministrationClient)
FormTrainingClientPreparer = functools.partial(_GlobalClientPreparer, FormTrainingClient)


class TestSendRequest(FormRecognizerTest):
    @FormRecognizerPreparer()
    @DocumentModelAdministrationClientPreparer()
    @recorded_by_proxy
    def test_get_resource_details(self, client, **kwargs):
        set_bodiless_matcher()
        request = HttpRequest(
            method="GET",
            url="info",
            headers={"Accept": "application/json"},
        )
        result = client.send_request(request)
        received_info1 = result.json()
        assert received_info1
        assert received_info1["customDocumentModels"]
        assert received_info1["customDocumentModels"]["count"] == 0
        assert received_info1["customDocumentModels"]["limit"] == 250

        request = HttpRequest(
            method="GET",
            url="info?api-version=2022-08-31",
            headers={"Accept": "application/json"},
        )
        result = client.send_request(request)
        received_info2 = result.json()
        assert received_info2["customDocumentModels"]["count"] == received_info1["customDocumentModels"]["count"]
        assert received_info2["customDocumentModels"]["limit"] == received_info1["customDocumentModels"]["limit"]
        
        # test with absolute url
        request = HttpRequest(
            method="GET",
            url=f"{client._endpoint}/formrecognizer/info?api-version=2022-08-31",
            headers={"Accept": "application/json"},
        )
        result = client.send_request(request)
        received_info3 = result.json()
        assert received_info3["customDocumentModels"]["count"] == received_info1["customDocumentModels"]["count"]
        assert received_info3["customDocumentModels"]["limit"] == received_info1["customDocumentModels"]["limit"]
        
        # test with v2 API version
        request = HttpRequest(
            method="GET",
            url="v2.1/custom/models?op=summary",
            headers={"Accept": "application/json"},
        )
        result = client.send_request(request)
        received_info4 = result.json()
        assert received_info4
        assert received_info4["summary"]["count"] == 0
        assert received_info4["summary"]["limit"]
        
        # test with v2 API version with absolute url
        request = HttpRequest(
            method="GET",
            url=f"{client._endpoint}/formrecognizer/v2.1/custom/models?op=summary",
            headers={"Accept": "application/json"},
        )
        result = client.send_request(request)
        received_info5 = result.json()
        assert received_info5["summary"]["count"] == received_info4["summary"]["count"]
        assert received_info5["summary"]["limit"] == received_info4["summary"]["limit"]

    @FormRecognizerPreparer()
    @FormTrainingClientPreparer(client_kwargs={"api_version": "2.1"})
    @recorded_by_proxy
    def test_get_account_properties_v2(self, client):
        set_bodiless_matcher()
        request = HttpRequest(
            method="GET",
            url="custom/models?op=summary",
            headers={"Accept": "application/json"},
        )
        result = client.send_request(request)
        received_info1 = result.json()
        assert received_info1
        assert received_info1["summary"]
        assert received_info1["summary"]["count"] == 0
        assert received_info1["summary"]["limit"] == 250
        
        # test with absolute url
        request = HttpRequest(
            method="GET",
            url=f"{client._endpoint}/formrecognizer/v2.0/custom/models?op=summary",
            headers={"Accept": "application/json"},
        )
        result = client.send_request(request)
        received_info3 = result.json()
        assert received_info3["summary"]["count"] == received_info1["summary"]["count"]
        assert received_info3["summary"]["limit"] == received_info1["summary"]["limit"]
        
        # relative URLs can't override the API version on 2.x clients
        request = HttpRequest(
            method="GET",
            url="info?api-version=2022-08-31",
            headers={"Accept": "application/json"},
        )
        result = client.send_request(request)
        received_info4 = result.json()
        assert received_info4["error"]["code"] == "404"
        assert received_info4["error"]["message"] == "Resource not found"
        
        # test with v2 API version with absolute url
        request = HttpRequest(
            method="GET",
            url=f"{client._endpoint}/formrecognizer/info?api-version=2022-08-31",
            headers={"Accept": "application/json"},
        )
        result = client.send_request(request)
        received_info5 = result.json()
        assert received_info5
        assert received_info5["customDocumentModels"]["count"] == 0
        assert received_info5["customDocumentModels"]["limit"] == 250



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

import pytest
import functools
from devtools_testutils.aio import recorded_by_proxy_async
from azure.core.exceptions import HttpResponseError
from azure.ai.formrecognizer.aio import FormTrainingClient
from azure.ai.formrecognizer import CustomFormModel
from preparers import FormRecognizerPreparer
from preparers import GlobalClientPreparer as _GlobalClientPreparer
from asynctestcase import AsyncFormRecognizerTest
from conftest import skip_flaky_test


FormTrainingClientPreparer = functools.partial(_GlobalClientPreparer, FormTrainingClient)


class TestTrainingAsync(AsyncFormRecognizerTest):

    @skip_flaky_test
    @FormRecognizerPreparer()
    @FormTrainingClientPreparer(client_kwargs={"api_version": "2.1"})
    @recorded_by_proxy_async
    async def test_compose_model_v21(self, client, formrecognizer_storage_container_sas_url_v2, **kwargs):
        async with client:
            poller = await client.begin_training(formrecognizer_storage_container_sas_url_v2, use_training_labels=True)
            model_1 = await poller.result()

            poller = await client.begin_training(formrecognizer_storage_container_sas_url_v2, use_training_labels=True, model_name="second-labeled-model")
            model_2 = await poller.result()

            poller = await client.begin_create_composed_model([model_1.model_id, model_2.model_id], model_name="my composed model")

            composed_model = await poller.result()

        composed_model_dict = composed_model.to_dict()
        composed_model = CustomFormModel.from_dict(composed_model_dict)
        self.assertComposedModelV2HasValues(composed_model, model_1, model_2)

    @FormRecognizerPreparer()
    @FormTrainingClientPreparer(client_kwargs={"api_version": "2.1"})
    @recorded_by_proxy_async
    async def test_compose_model_invalid_unlabeled_models_v21(self, client, formrecognizer_storage_container_sas_url_v2, **kwargs):
        async with client:
            poller = await client.begin_training(formrecognizer_storage_container_sas_url_v2, use_training_labels=False)
            model_1 = await poller.result()

            poller = await client.begin_training(formrecognizer_storage_container_sas_url_v2, use_training_labels=False)
            model_2 = await poller.result()

            with pytest.raises(HttpResponseError) as e:
                poller = await client.begin_create_composed_model([model_1.model_id, model_2.model_id])
                composed_model = await poller.result()
            assert e.value.error.code == "1001"
            assert e.value.error.message

    @FormRecognizerPreparer()
    @FormTrainingClientPreparer(client_kwargs={"api_version": "2.0"})
    async def test_compose_model_bad_api_version(self, **kwargs):
        client = kwargs.pop("client")
        async with client:
            with pytest.raises(ValueError) as excinfo:
                poller = await client.begin_create_composed_model(["00000000-0000-0000-0000-000000000000", "00000000-0000-0000-0000-000000000000"])
                result = await poller.result()
            assert "Method 'begin_create_composed_model' is only available for API version V2_1 and up" in str(excinfo.value)



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

import pytest
import functools
from devtools_testutils.aio import recorded_by_proxy_async
from azure.ai.formrecognizer.aio import FormTrainingClient
from azure.ai.formrecognizer._generated.v2_1.models import AnalyzeOperationResult
from azure.ai.formrecognizer._response_handlers import prepare_form_result
from preparers import FormRecognizerPreparer
from asynctestcase import AsyncFormRecognizerTest
from testcase import _get_blob_url
from preparers import GlobalClientPreparer as _GlobalClientPreparer
from conftest import skip_flaky_test
FormTrainingClientPreparer = functools.partial(_GlobalClientPreparer, FormTrainingClient)


class TestCustomFormsFromUrlAsync(AsyncFormRecognizerTest):

    @skip_flaky_test
    @FormRecognizerPreparer()
    @FormTrainingClientPreparer()
    @recorded_by_proxy_async
    async def test_custom_form_multipage_unlabeled(self, client, formrecognizer_multipage_storage_container_sas_url_v2, **kwargs):
        fr_client = client.get_form_recognizer_client()
        blob_sas_url = _get_blob_url(formrecognizer_multipage_storage_container_sas_url_v2, "multipage-training-data", "multipage_invoice1.pdf")
        async with client:
            training_poller = await client.begin_training(formrecognizer_multipage_storage_container_sas_url_v2, use_training_labels=False)
            model = await training_poller.result()

            async with fr_client:
                poller = await fr_client.begin_recognize_custom_forms_from_url(
                    model.model_id,
                    blob_sas_url,
                )
                forms = await poller.result()

        for form in forms:
            if form.form_type is None:
                continue  # blank page
            assert form.form_type == "form-0"
            self.assertUnlabeledRecognizedFormHasValues(form, model)

    @skip_flaky_test
    @FormRecognizerPreparer()
    @FormTrainingClientPreparer()
    @recorded_by_proxy_async
    async def test_form_multipage_labeled(self, client, formrecognizer_multipage_storage_container_sas_url_v2, **kwargs):
        fr_client = client.get_form_recognizer_client()
        blob_sas_url = _get_blob_url(formrecognizer_multipage_storage_container_sas_url_v2, "multipage-training-data", "multipage_invoice1.pdf")
        async with client:
            training_poller = await client.begin_training(
                formrecognizer_multipage_storage_container_sas_url_v2,
                use_training_labels=True
            )
            model = await training_poller.result()

            async with fr_client:
                poller = await fr_client.begin_recognize_custom_forms_from_url(
                    model.model_id,
                    blob_sas_url
                )
                forms = await poller.result()

        for form in forms:
            assert form.form_type ==  "custom:"+model.model_id
            self.assertLabeledRecognizedFormHasValues(form, model)

    @skip_flaky_test
    @FormRecognizerPreparer()
    @FormTrainingClientPreparer()
    @recorded_by_proxy_async
    async def test_multipage_unlabeled_transform(self, client, formrecognizer_multipage_storage_container_sas_url_v2, **kwargs):
        fr_client = client.get_form_recognizer_client()
        blob_sas_url = _get_blob_url(formrecognizer_multipage_storage_container_sas_url_v2, "multipage-training-data", "multipage_invoice1.pdf")
        responses = []

        def callback(raw_response, _, headers):
            analyze_result = fr_client._deserialize(AnalyzeOperationResult, raw_response)
            form = prepare_form_result(analyze_result, model.model_id)
            responses.append(analyze_result)
            responses.append(form)

        async with client:
            training_poller = await client.begin_training(formrecognizer_multipage_storage_container_sas_url_v2, use_training_labels=False)
            model = await training_poller.result()

            async with fr_client:
                poller = await fr_client.begin_recognize_custom_forms_from_url(
                    model.model_id,
                    blob_sas_url,
                    include_field_elements=True,
                    cls=callback
                )

                form = await poller.result()
        actual = responses[0]
        recognized_form = responses[1]
        read_results = actual.analyze_result.read_results
        page_results = actual.analyze_result.page_results

        self.assertFormPagesTransformCorrect(recognized_form, read_results, page_results)

        for form, actual in zip(recognized_form, page_results):
            assert form.page_range.first_page_number ==  actual.page
            assert form.page_range.last_page_number ==  actual.page
            assert form.form_type_confidence is None
            assert form.model_id ==  model.model_id
            self.assertUnlabeledFormFieldDictTransformCorrect(form.fields, actual.key_value_pairs, read_results)

    @skip_flaky_test
    @FormRecognizerPreparer()
    @FormTrainingClientPreparer()
    @recorded_by_proxy_async
    async def test_multipage_labeled_transform(self, client, formrecognizer_multipage_storage_container_sas_url_v2, **kwargs):
        fr_client = client.get_form_recognizer_client()
        blob_sas_url = _get_blob_url(formrecognizer_multipage_storage_container_sas_url_v2, "multipage-training-data", "multipage_invoice1.pdf")
        responses = []

        def callback(raw_response, _, headers):
            analyze_result = fr_client._deserialize(AnalyzeOperationResult, raw_response)
            form = prepare_form_result(analyze_result, model.model_id)
            responses.append(analyze_result)
            responses.append(form)

        async with client:
            training_poller = await client.begin_training(formrecognizer_multipage_storage_container_sas_url_v2, use_training_labels=True)
            model = await training_poller.result()

            async with fr_client:
                poller = await fr_client.begin_recognize_custom_forms_from_url(
                    model.model_id,
                    blob_sas_url,
                    include_field_elements=True,
                    cls=callback
                )
                form = await poller.result()

        actual = responses[0]
        recognized_form = responses[1]
        read_results = actual.analyze_result.read_results
        page_results = actual.analyze_result.page_results
        document_results = actual.analyze_result.document_results

        self.assertFormPagesTransformCorrect(recognized_form, read_results, page_results)
        for form, actual in zip(recognized_form, document_results):
            assert form.page_range.first_page_number ==  actual.page_range[0]
            assert form.page_range.last_page_number ==  actual.page_range[1]
            assert form.form_type ==  "custom:"+model.model_id
            assert form.form_type_confidence is not None
            assert form.model_id ==  model.model_id
            self.assertFormFieldsTransformCorrect(form.fields, actual.fields, read_results)

    @pytest.mark.live_test_only
    @skip_flaky_test
    @FormRecognizerPreparer()
    @FormTrainingClientPreparer()
    async def test_custom_form_continuation_token(self, **kwargs):
        client = kwargs.pop("client")
        formrecognizer_storage_container_sas_url = kwargs.pop("formrecognizer_storage_container_sas_url")
        fr_client = client.get_form_recognizer_client()

        async with client:
            poller = await client.begin_training(formrecognizer_storage_container_sas_url, use_training_labels=False)
            model = await poller.result()

            async with fr_client:
                initial_poller = await fr_client.begin_recognize_custom_forms_from_url(
                    model.model_id,
                    self.form_url_jpg
                )
                cont_token = initial_poller.continuation_token()
                poller = await fr_client.begin_recognize_custom_forms_from_url(
                    model.model_id,
                    None,
                    continuation_token=cont_token
                )
                result = await poller.result()
                assert result is not None
                await initial_poller.wait()  # necessary so azure-devtools doesn't throw assertion error

    @skip_flaky_test
    @FormRecognizerPreparer()
    @FormTrainingClientPreparer()
    @recorded_by_proxy_async
    async def test_custom_form_multipage_vendor_set_unlabeled_transform(self, client, formrecognizer_multipage_storage_container_sas_url_2_v2, **kwargs):
        fr_client = client.get_form_recognizer_client()
        blob_sas_url = _get_blob_url(formrecognizer_multipage_storage_container_sas_url_2_v2, "multipage-vendor-forms", "multi1.pdf")
        responses = []

        def callback(raw_response, _, headers):
            analyze_result = fr_client._deserialize(AnalyzeOperationResult, raw_response)
            form = prepare_form_result(analyze_result, model.model_id)
            responses.append(analyze_result)
            responses.append(form)

        async with client:
            poller = await client.begin_training(formrecognizer_multipage_storage_container_sas_url_2_v2, use_training_labels=False)
            model = await poller.result()

            async with fr_client:
                poller = await fr_client.begin_recognize_custom_forms_from_url(
                    model.model_id,
                    blob_sas_url,
                    include_field_elements=True,
                    cls=callback
                )
                form = await poller.result()
        actual = responses[0]
        recognized_form = responses[1]
        read_results = actual.analyze_result.read_results
        page_results = actual.analyze_result.page_results

        self.assertFormPagesTransformCorrect(recognized_form, read_results, page_results)
        for form, actual in zip(recognized_form, page_results):
            assert form.page_range.first_page_number ==  actual.page
            assert form.page_range.last_page_number ==  actual.page
            assert form.form_type_confidence is None
            assert form.model_id ==  model.model_id
            self.assertUnlabeledFormFieldDictTransformCorrect(form.fields, actual.key_value_pairs, read_results)

    @skip_flaky_test
    @FormRecognizerPreparer()
    @FormTrainingClientPreparer()
    @recorded_by_proxy_async
    async def test_custom_form_multipage_vendor_set_labeled_transform(self, client, formrecognizer_multipage_storage_container_sas_url_2_v2, **kwargs):
        fr_client = client.get_form_recognizer_client()
        blob_sas_url = _get_blob_url(formrecognizer_multipage_storage_container_sas_url_2_v2, "multipage-vendor-forms", "multi1.pdf")
        responses = []

        def callback(raw_response, _, headers):
            analyze_result = fr_client._deserialize(AnalyzeOperationResult, raw_response)
            form = prepare_form_result(analyze_result, model.model_id)
            responses.append(analyze_result)
            responses.append(form)

        async with client:
            poller = await client.begin_training(formrecognizer_multipage_storage_container_sas_url_2_v2, use_training_labels=True)
            model = await poller.result()

            async with fr_client:
                poller = await fr_client.begin_recognize_custom_forms_from_url(
                    model.model_id,
                    blob_sas_url,
                    include_field_elements=True,
                    cls=callback
                )
                form = await poller.result()
        actual = responses[0]
        recognized_form = responses[1]
        read_results = actual.analyze_result.read_results
        page_results = actual.analyze_result.page_results
        document_results = actual.analyze_result.document_results

        self.assertFormPagesTransformCorrect(recognized_form, read_results, page_results)
        for form, actual in zip(recognized_form, document_results):
            assert form.page_range.first_page_number ==  actual.page_range[0]
            assert form.page_range.last_page_number ==  actual.page_range[1]
            assert form.form_type ==  "custom:"+model.model_id
            assert form.form_type_confidence is not None
            assert form.model_id ==  model.model_id
            self.assertFormFieldsTransformCorrect(form.fields, actual.fields, read_results)

    @skip_flaky_test
    @FormRecognizerPreparer()
    @FormTrainingClientPreparer()
    @recorded_by_proxy_async
    async def test_pages_kwarg_specified(self, client, formrecognizer_testing_data_container_sas_url, **kwargs):
        fr_client = client.get_form_recognizer_client()
        blob_sas_url = _get_blob_url(formrecognizer_testing_data_container_sas_url, "testingdata", "multi1.pdf")

        async with fr_client:
            training_poller = await client.begin_training(formrecognizer_testing_data_container_sas_url, use_training_labels=False)
            model = await training_poller.result()

            poller = await fr_client.begin_recognize_custom_forms_from_url(
                model.model_id, 
                blob_sas_url, 
                pages=["1"])
            
            assert '1' == poller._polling_method._initial_response.http_response.request.query['pages']
            result = await poller.result()
            assert result



# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

import pytest
from azure.ai.formrecognizer._helpers import get_content_type
from testcase import FormRecognizerTest


class TestContentType(FormRecognizerTest):

    def test_pdf(self):
        with open(self.invoice_pdf, "rb") as fd:
            content_type = get_content_type(fd)
        assert content_type == "application/pdf"

    def test_pdf_bytes(self):
        with open(self.invoice_pdf, "rb") as fd:
            my_file = fd.read()
        content_type = get_content_type(my_file)
        assert content_type == "application/pdf"

    def test_jpg(self):
        with open(self.form_jpg, "rb") as fd:
            content_type = get_content_type(fd)
        assert content_type == "image/jpeg"

    def test_jpg_bytes(self):
        with open(self.form_jpg, "rb") as fd:
            my_file = fd.read()
        content_type = get_content_type(my_file)
        assert content_type == "image/jpeg"

    def test_png(self):
        with open(self.receipt_png, "rb") as fd:
            content_type = get_content_type(fd)
        assert content_type == "image/png"

    def test_png_bytes(self):
        with open(self.receipt_png, "rb") as fd:
            my_file = fd.read()
        content_type = get_content_type(my_file)
        assert content_type == "image/png"

    def test_tiff_little_endian(self):
        with open(self.invoice_tiff, "rb") as fd:
            content_type = get_content_type(fd)
        assert content_type == "image/tiff"

    def test_tiff_little_endian_bytes(self):
        with open(self.invoice_tiff, "rb") as fd:
            my_file = fd.read()
        content_type = get_content_type(my_file)
        assert content_type == "image/tiff"

    def test_tiff_big_endian(self):
        content_type = get_content_type(b"\x4D\x4D\x00\x2A")
        assert content_type == "image/tiff"

    def test_bmp(self):
        content_type = get_content_type(b"\x42\x4D\x00\x00")
        assert content_type == "image/bmp"



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: sample_convert_to_and_from_dict_async.py

DESCRIPTION:
    This sample demonstrates how to convert models returned from an analyze operation
    to and from a dictionary. The dictionary in this sample is then converted to a
    JSON file, then the same dictionary is converted back to its original model.

USAGE:
    python sample_convert_to_and_from_dict_async.py

    Set the environment variables with your own values before running the sample:
    1) AZURE_FORM_RECOGNIZER_ENDPOINT - the endpoint to your Form Recognizer resource.
    2) AZURE_FORM_RECOGNIZER_KEY - your Form Recognizer API key
"""

import os
import json
import asyncio


async def convert_to_and_from_dict_async():
    path_to_sample_documents = os.path.abspath(
        os.path.join(
            os.path.abspath(__file__),
            "..",
            "..",
            "..",
            "./sample_forms/forms/Form_1.jpg",
        )
    )

    from azure.core.serialization import AzureJSONEncoder
    from azure.core.credentials import AzureKeyCredential
    from azure.ai.formrecognizer.aio import DocumentAnalysisClient
    from azure.ai.formrecognizer import AnalyzeResult

    endpoint = os.environ["AZURE_FORM_RECOGNIZER_ENDPOINT"]
    key = os.environ["AZURE_FORM_RECOGNIZER_KEY"]

    document_analysis_client = DocumentAnalysisClient(
        endpoint=endpoint, credential=AzureKeyCredential(key)
    )
    async with document_analysis_client:
        with open(path_to_sample_documents, "rb") as f:
            poller = await document_analysis_client.begin_analyze_document(
                "prebuilt-document", document=f
            )
        result = await poller.result()

    # convert the received model to a dictionary
    analyze_result_dict = result.to_dict()

    # save the dictionary as JSON content in a JSON file, use the AzureJSONEncoder
    # to help make types, such as dates, JSON serializable
    # NOTE: AzureJSONEncoder is only available with azure.core>=1.18.0.
    with open("data.json", "w") as output_file:
        json.dump(analyze_result_dict, output_file, cls=AzureJSONEncoder)

    # convert the dictionary back to the original model
    model = AnalyzeResult.from_dict(analyze_result_dict)

    # use the model as normal
    print("----Converted from dictionary AnalyzeResult----")
    print(f"Model ID: '{model.model_id}'")
    print(f"Number of pages analyzed {len(model.pages)}")
    print(f"API version used: {model.api_version}")

    print("----------------------------------------")


async def main():
    await convert_to_and_from_dict_async()


if __name__ == "__main__":
    import sys
    from azure.core.exceptions import HttpResponseError

    try:
        asyncio.run(main())
    except HttpResponseError as error:
        print(
            "For more information about troubleshooting errors, see the following guide: "
            "https://aka.ms/azsdk/python/formrecognizer/troubleshooting"
        )
        # Examples of how to check an HttpResponseError
        # Check by error code:
        if error.error is not None:
            if error.error.code == "InvalidImage":
                print(f"Received an invalid image error: {error.error}")
            if error.error.code == "InvalidRequest":
                print(f"Received an invalid request error: {error.error}")
            # Raise the error again after printing it
            raise
        # If the inner error is None and then it is possible to check the message to get more information:
        if "Invalid request".casefold() in error.message.casefold():
            print(f"Uh-oh! Seems there was an invalid request: {error}")
        # Raise the error again
        raise



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: sample_recognize_receipts_from_url.py

DESCRIPTION:
    This sample demonstrates how to recognize and extract common fields from a receipt URL,
    using a pre-trained receipt model. For a suggested approach to extracting information
    from receipts, see sample_strongly_typed_recognized_form.py.

    See fields found on a receipt here:
    https://aka.ms/formrecognizer/receiptfields

USAGE:
    python sample_recognize_receipts_from_url.py

    Set the environment variables with your own values before running the sample:
    1) AZURE_FORM_RECOGNIZER_ENDPOINT - the endpoint to your Form Recognizer resource.
    2) AZURE_FORM_RECOGNIZER_KEY - your Form Recognizer API key
"""

import os


class RecognizeReceiptsFromURLSample(object):

    def recognize_receipts_from_url(self):
        # [START recognize_receipts_from_url]
        from azure.core.credentials import AzureKeyCredential
        from azure.ai.formrecognizer import FormRecognizerClient

        endpoint = os.environ["AZURE_FORM_RECOGNIZER_ENDPOINT"]
        key = os.environ["AZURE_FORM_RECOGNIZER_KEY"]

        form_recognizer_client = FormRecognizerClient(
            endpoint=endpoint, credential=AzureKeyCredential(key)
        )
        url = "https://raw.githubusercontent.com/Azure/azure-sdk-for-python/main/sdk/formrecognizer/azure-ai-formrecognizer/tests/sample_forms/receipt/contoso-receipt.png"
        poller = form_recognizer_client.begin_recognize_receipts_from_url(receipt_url=url)
        receipts = poller.result()

        for idx, receipt in enumerate(receipts):
            print("--------Recognizing receipt #{}--------".format(idx+1))
            receipt_type = receipt.fields.get("ReceiptType")
            if receipt_type:
                print("Receipt Type: {} has confidence: {}".format(receipt_type.value, receipt_type.confidence))
            merchant_name = receipt.fields.get("MerchantName")
            if merchant_name:
                print("Merchant Name: {} has confidence: {}".format(merchant_name.value, merchant_name.confidence))
            transaction_date = receipt.fields.get("TransactionDate")
            if transaction_date:
                print("Transaction Date: {} has confidence: {}".format(transaction_date.value, transaction_date.confidence))
            if receipt.fields.get("Items"):
                print("Receipt items:")
                for idx, item in enumerate(receipt.fields.get("Items").value):
                    print("...Item #{}".format(idx+1))
                    item_name = item.value.get("Name")
                    if item_name:
                        print("......Item Name: {} has confidence: {}".format(item_name.value, item_name.confidence))
                    item_quantity = item.value.get("Quantity")
                    if item_quantity:
                        print("......Item Quantity: {} has confidence: {}".format(item_quantity.value, item_quantity.confidence))
                    item_price = item.value.get("Price")
                    if item_price:
                        print("......Individual Item Price: {} has confidence: {}".format(item_price.value, item_price.confidence))
                    item_total_price = item.value.get("TotalPrice")
                    if item_total_price:
                        print("......Total Item Price: {} has confidence: {}".format(item_total_price.value, item_total_price.confidence))
            subtotal = receipt.fields.get("Subtotal")
            if subtotal:
                print("Subtotal: {} has confidence: {}".format(subtotal.value, subtotal.confidence))
            tax = receipt.fields.get("Tax")
            if tax:
                print("Tax: {} has confidence: {}".format(tax.value, tax.confidence))
            tip = receipt.fields.get("Tip")
            if tip:
                print("Tip: {} has confidence: {}".format(tip.value, tip.confidence))
            total = receipt.fields.get("Total")
            if total:
                print("Total: {} has confidence: {}".format(total.value, total.confidence))
            print("--------------------------------------")
        # [END recognize_receipts_from_url]


if __name__ == '__main__':
    sample = RecognizeReceiptsFromURLSample()
    sample.recognize_receipts_from_url()




# coding: utf-8
# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import logging
import pytest
from functools import wraps
from azure.core.exceptions import HttpResponseError
import sys
from devtools_testutils import (
    add_remove_header_sanitizer,
    add_general_regex_sanitizer,
    add_oauth_response_sanitizer,
    add_body_key_sanitizer,
    test_proxy,
)

@pytest.fixture(scope="session", autouse=True)
def add_sanitizers(test_proxy):
    add_remove_header_sanitizer(headers="Ocp-Apim-Subscription-Key")
    add_general_regex_sanitizer(
        value="fakeendpoint",
        regex="(?<=\\/\\/)[a-z-]+(?=\\.cognitiveservices\\.azure\\.com)"
    )
    add_oauth_response_sanitizer()
    add_body_key_sanitizer(
        json_path="urlSource",
        value="blob_sas_url",
        regex="(?<=\\/\\/)[a-z-]+(?=\\.blob\\.core\\.windows\\.net)(.*)$",
    )
    add_body_key_sanitizer(
        json_path="azureBlobSource.containerUrl",
        value="blob_sas_url",
        regex="(?<=\\/\\/)[a-z-]+(?=\\.blob\\.core\\.windows\\.net)(.*)$",
    )
    add_body_key_sanitizer(
        json_path="source",
        value="blob_sas_url",
        regex="(?<=\\/\\/)[a-z-]+(?=\\.blob\\.core\\.windows\\.net)(.*)$",
    )
    add_body_key_sanitizer(
        json_path="accessToken",
        value="redacted",
        regex="([0-9a-f-]{36})",
    )
    add_body_key_sanitizer(
        json_path="targetResourceId",
        value="/path/to/resource/id",
        regex="^.*",
    )
    add_body_key_sanitizer(
        json_path="targetResourceRegion",
        value="region",
        regex="^.*",
    )
    add_body_key_sanitizer(
        json_path="copyAuthorization.accessToken",
        value="redacted",
        regex="([0-9a-f-]{36})",
    )

def skip_flaky_test(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except HttpResponseError as error:
            logger = logging.getLogger("azure")
            if "Invalid request".casefold() in error.message.casefold():
                pytest.mark.skip("flaky service response: {}".format(error))
                logger.debug("flaky service response: {}".format(error))
            elif "Generic error".casefold() in error.message.casefold():
                pytest.mark.skip("flaky service response: {}".format(error))
                logger.debug("flaky service response: {}".format(error))
            elif "Timeout" in error.message.casefold():
                pytest.mark.skip("flaky service response: {}".format(error))
                logger.debug("flaky service response: {}".format(error))
            elif "InvalidImage" in error.message.casefold():
                pytest.mark.skip("flaky service response: {}".format(error))
                logger.debug("flaky service response: {}".format(error))
            else:
                raise  # not a flaky test

    return wrapper



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: sample_build_model_async.py

DESCRIPTION:
    This sample demonstrates how to build a model. For this sample, you can use the training
    documents found in https://aka.ms/azsdk/formrecognizer/sampletrainingfiles

    More details on setting up a container and required file structure can be found here:
    https://aka.ms/azsdk/formrecognizer/buildtrainingset

USAGE:
    python sample_build_model_async.py

    Set the environment variables with your own values before running the sample:
    1) AZURE_FORM_RECOGNIZER_ENDPOINT - the endpoint to your Form Recognizer resource.
    2) AZURE_FORM_RECOGNIZER_KEY - your Form Recognizer API key
    3) CONTAINER_SAS_URL - The shared access signature (SAS) Url of your Azure Blob Storage container with your training files.
"""

import os
import asyncio


async def sample_build_model_async():
    # [START build_model_async]
    from azure.ai.formrecognizer.aio import DocumentModelAdministrationClient
    from azure.ai.formrecognizer import ModelBuildMode
    from azure.core.credentials import AzureKeyCredential

    endpoint = os.environ["AZURE_FORM_RECOGNIZER_ENDPOINT"]
    key = os.environ["AZURE_FORM_RECOGNIZER_KEY"]
    container_sas_url = os.environ["CONTAINER_SAS_URL"]

    document_model_admin_client = DocumentModelAdministrationClient(
        endpoint, AzureKeyCredential(key)
    )
    async with document_model_admin_client:
        poller = await document_model_admin_client.begin_build_document_model(
            ModelBuildMode.TEMPLATE,
            blob_container_url=container_sas_url,
            description="my model description",
        )
        model = await poller.result()

    print(f"Model ID: {model.model_id}")
    print(f"Description: {model.description}")
    print(f"Model created on: {model.created_on}")
    print(f"Model expires on: {model.expires_on}")
    print("Doc types the model can recognize:")
    for name, doc_type in model.doc_types.items():
        print(
            f"Doc Type: '{name}' built with '{doc_type.build_mode}' mode which has the following fields:"
        )
        for field_name, field in doc_type.field_schema.items():
            print(
                f"Field: '{field_name}' has type '{field['type']}' and confidence score "
                f"{doc_type.field_confidence[field_name]}"
            )
    # [END build_model_async]


async def main():
    await sample_build_model_async()


if __name__ == "__main__":
    import sys
    from azure.core.exceptions import HttpResponseError

    try:
        asyncio.run(main())
    except HttpResponseError as error:
        print(
            "For more information about troubleshooting errors, see the following guide: "
            "https://aka.ms/azsdk/python/formrecognizer/troubleshooting"
        )
        # Examples of how to check an HttpResponseError
        # Check by error code:
        if error.error is not None:
            if error.error.code == "InvalidImage":
                print(f"Received an invalid image error: {error.error}")
            if error.error.code == "InvalidRequest":
                print(f"Received an invalid request error: {error.error}")
            # Raise the error again after printing it
            raise
        # If the inner error is None and then it is possible to check the message to get more information:
        if "Invalid request".casefold() in error.message.casefold():
            print(f"Uh-oh! Seems there was an invalid request: {error}")
        # Raise the error again
        raise



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

import datetime
from typing import Dict, List, Optional, TYPE_CHECKING, Union

from azure.core.exceptions import HttpResponseError
import msrest.serialization

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    import __init__ as _models


class AnalyzeOperationResult(msrest.serialization.Model):
    """Status and result of the queued analyze operation.

    All required parameters must be populated in order to send to Azure.

    :ivar status: Required. Operation status. Known values are: "notStarted", "running",
     "succeeded", "failed".
    :vartype status: str or ~azure.ai.formrecognizer.v2_0.models.OperationStatus
    :ivar created_date_time: Required. Date and time (UTC) when the analyze operation was
     submitted.
    :vartype created_date_time: ~datetime.datetime
    :ivar last_updated_date_time: Required. Date and time (UTC) when the status was last updated.
    :vartype last_updated_date_time: ~datetime.datetime
    :ivar analyze_result: Results of the analyze operation.
    :vartype analyze_result: ~azure.ai.formrecognizer.v2_0.models.AnalyzeResult
    """

    _validation = {
        'status': {'required': True},
        'created_date_time': {'required': True},
        'last_updated_date_time': {'required': True},
    }

    _attribute_map = {
        'status': {'key': 'status', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_updated_date_time': {'key': 'lastUpdatedDateTime', 'type': 'iso-8601'},
        'analyze_result': {'key': 'analyzeResult', 'type': 'AnalyzeResult'},
    }

    def __init__(
        self,
        *,
        status: Union[str, "_models.OperationStatus"],
        created_date_time: datetime.datetime,
        last_updated_date_time: datetime.datetime,
        analyze_result: Optional["_models.AnalyzeResult"] = None,
        **kwargs
    ):
        """
        :keyword status: Required. Operation status. Known values are: "notStarted", "running",
         "succeeded", "failed".
        :paramtype status: str or ~azure.ai.formrecognizer.v2_0.models.OperationStatus
        :keyword created_date_time: Required. Date and time (UTC) when the analyze operation was
         submitted.
        :paramtype created_date_time: ~datetime.datetime
        :keyword last_updated_date_time: Required. Date and time (UTC) when the status was last
         updated.
        :paramtype last_updated_date_time: ~datetime.datetime
        :keyword analyze_result: Results of the analyze operation.
        :paramtype analyze_result: ~azure.ai.formrecognizer.v2_0.models.AnalyzeResult
        """
        super(AnalyzeOperationResult, self).__init__(**kwargs)
        self.status = status
        self.created_date_time = created_date_time
        self.last_updated_date_time = last_updated_date_time
        self.analyze_result = analyze_result


class AnalyzeResult(msrest.serialization.Model):
    """Analyze operation result.

    All required parameters must be populated in order to send to Azure.

    :ivar version: Required. Version of schema used for this result.
    :vartype version: str
    :ivar read_results: Required. Text extracted from the input.
    :vartype read_results: list[~azure.ai.formrecognizer.v2_0.models.ReadResult]
    :ivar page_results: Page-level information extracted from the input.
    :vartype page_results: list[~azure.ai.formrecognizer.v2_0.models.PageResult]
    :ivar document_results: Document-level information extracted from the input.
    :vartype document_results: list[~azure.ai.formrecognizer.v2_0.models.DocumentResult]
    :ivar errors: List of errors reported during the analyze operation.
    :vartype errors: list[~azure.ai.formrecognizer.v2_0.models.ErrorInformation]
    """

    _validation = {
        'version': {'required': True},
        'read_results': {'required': True},
    }

    _attribute_map = {
        'version': {'key': 'version', 'type': 'str'},
        'read_results': {'key': 'readResults', 'type': '[ReadResult]'},
        'page_results': {'key': 'pageResults', 'type': '[PageResult]'},
        'document_results': {'key': 'documentResults', 'type': '[DocumentResult]'},
        'errors': {'key': 'errors', 'type': '[ErrorInformation]'},
    }

    def __init__(
        self,
        *,
        version: str,
        read_results: List["_models.ReadResult"],
        page_results: Optional[List["_models.PageResult"]] = None,
        document_results: Optional[List["_models.DocumentResult"]] = None,
        errors: Optional[List["_models.ErrorInformation"]] = None,
        **kwargs
    ):
        """
        :keyword version: Required. Version of schema used for this result.
        :paramtype version: str
        :keyword read_results: Required. Text extracted from the input.
        :paramtype read_results: list[~azure.ai.formrecognizer.v2_0.models.ReadResult]
        :keyword page_results: Page-level information extracted from the input.
        :paramtype page_results: list[~azure.ai.formrecognizer.v2_0.models.PageResult]
        :keyword document_results: Document-level information extracted from the input.
        :paramtype document_results: list[~azure.ai.formrecognizer.v2_0.models.DocumentResult]
        :keyword errors: List of errors reported during the analyze operation.
        :paramtype errors: list[~azure.ai.formrecognizer.v2_0.models.ErrorInformation]
        """
        super(AnalyzeResult, self).__init__(**kwargs)
        self.version = version
        self.read_results = read_results
        self.page_results = page_results
        self.document_results = document_results
        self.errors = errors


class CopyAuthorizationResult(msrest.serialization.Model):
    """Request parameter that contains authorization claims for copy operation.

    All required parameters must be populated in order to send to Azure.

    :ivar model_id: Required. Model identifier.
    :vartype model_id: str
    :ivar access_token: Required. Token claim used to authorize the request.
    :vartype access_token: str
    :ivar expiration_date_time_ticks: Required. The time when the access token expires. The date is
     represented as the number of seconds from 1970-01-01T0:0:0Z UTC until the expiration time.
    :vartype expiration_date_time_ticks: long
    """

    _validation = {
        'model_id': {'required': True},
        'access_token': {'required': True},
        'expiration_date_time_ticks': {'required': True},
    }

    _attribute_map = {
        'model_id': {'key': 'modelId', 'type': 'str'},
        'access_token': {'key': 'accessToken', 'type': 'str'},
        'expiration_date_time_ticks': {'key': 'expirationDateTimeTicks', 'type': 'long'},
    }

    def __init__(
        self,
        *,
        model_id: str,
        access_token: str,
        expiration_date_time_ticks: int,
        **kwargs
    ):
        """
        :keyword model_id: Required. Model identifier.
        :paramtype model_id: str
        :keyword access_token: Required. Token claim used to authorize the request.
        :paramtype access_token: str
        :keyword expiration_date_time_ticks: Required. The time when the access token expires. The date
         is represented as the number of seconds from 1970-01-01T0:0:0Z UTC until the expiration time.
        :paramtype expiration_date_time_ticks: long
        """
        super(CopyAuthorizationResult, self).__init__(**kwargs)
        self.model_id = model_id
        self.access_token = access_token
        self.expiration_date_time_ticks = expiration_date_time_ticks


class CopyOperationResult(msrest.serialization.Model):
    """Status and result of the queued copy operation.

    All required parameters must be populated in order to send to Azure.

    :ivar status: Required. Operation status. Known values are: "notStarted", "running",
     "succeeded", "failed".
    :vartype status: str or ~azure.ai.formrecognizer.v2_0.models.OperationStatus
    :ivar created_date_time: Required. Date and time (UTC) when the copy operation was submitted.
    :vartype created_date_time: ~datetime.datetime
    :ivar last_updated_date_time: Required. Date and time (UTC) when the status was last updated.
    :vartype last_updated_date_time: ~datetime.datetime
    :ivar copy_result: Results of the copy operation.
    :vartype copy_result: ~azure.ai.formrecognizer.v2_0.models.CopyResult
    """

    _validation = {
        'status': {'required': True},
        'created_date_time': {'required': True},
        'last_updated_date_time': {'required': True},
    }

    _attribute_map = {
        'status': {'key': 'status', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_updated_date_time': {'key': 'lastUpdatedDateTime', 'type': 'iso-8601'},
        'copy_result': {'key': 'copyResult', 'type': 'CopyResult'},
    }

    def __init__(
        self,
        *,
        status: Union[str, "_models.OperationStatus"],
        created_date_time: datetime.datetime,
        last_updated_date_time: datetime.datetime,
        copy_result: Optional["_models.CopyResult"] = None,
        **kwargs
    ):
        """
        :keyword status: Required. Operation status. Known values are: "notStarted", "running",
         "succeeded", "failed".
        :paramtype status: str or ~azure.ai.formrecognizer.v2_0.models.OperationStatus
        :keyword created_date_time: Required. Date and time (UTC) when the copy operation was
         submitted.
        :paramtype created_date_time: ~datetime.datetime
        :keyword last_updated_date_time: Required. Date and time (UTC) when the status was last
         updated.
        :paramtype last_updated_date_time: ~datetime.datetime
        :keyword copy_result: Results of the copy operation.
        :paramtype copy_result: ~azure.ai.formrecognizer.v2_0.models.CopyResult
        """
        super(CopyOperationResult, self).__init__(**kwargs)
        self.status = status
        self.created_date_time = created_date_time
        self.last_updated_date_time = last_updated_date_time
        self.copy_result = copy_result


class CopyRequest(msrest.serialization.Model):
    """Request parameter to copy an existing custom model from the source resource to a target resource referenced by the resource ID.

    All required parameters must be populated in order to send to Azure.

    :ivar target_resource_id: Required. Azure Resource Id of the target Form Recognizer resource
     where the model is copied to.
    :vartype target_resource_id: str
    :ivar target_resource_region: Required. Location of the target Azure resource. A valid Azure
     region name supported by Cognitive Services.
    :vartype target_resource_region: str
    :ivar copy_authorization: Required. Entity that encodes claims to authorize the copy request.
    :vartype copy_authorization: ~azure.ai.formrecognizer.v2_0.models.CopyAuthorizationResult
    """

    _validation = {
        'target_resource_id': {'required': True, 'max_length': 1024, 'min_length': 0, 'pattern': r'^/subscriptions/[^/]*/resourceGroups/[^/]*/providers/Microsoft.CognitiveServices/accounts/[^/]*$'},
        'target_resource_region': {'required': True, 'max_length': 24, 'min_length': 1, 'pattern': r'^[a-z0-9]+$'},
        'copy_authorization': {'required': True},
    }

    _attribute_map = {
        'target_resource_id': {'key': 'targetResourceId', 'type': 'str'},
        'target_resource_region': {'key': 'targetResourceRegion', 'type': 'str'},
        'copy_authorization': {'key': 'copyAuthorization', 'type': 'CopyAuthorizationResult'},
    }

    def __init__(
        self,
        *,
        target_resource_id: str,
        target_resource_region: str,
        copy_authorization: "_models.CopyAuthorizationResult",
        **kwargs
    ):
        """
        :keyword target_resource_id: Required. Azure Resource Id of the target Form Recognizer resource
         where the model is copied to.
        :paramtype target_resource_id: str
        :keyword target_resource_region: Required. Location of the target Azure resource. A valid Azure
         region name supported by Cognitive Services.
        :paramtype target_resource_region: str
        :keyword copy_authorization: Required. Entity that encodes claims to authorize the copy
         request.
        :paramtype copy_authorization: ~azure.ai.formrecognizer.v2_0.models.CopyAuthorizationResult
        """
        super(CopyRequest, self).__init__(**kwargs)
        self.target_resource_id = target_resource_id
        self.target_resource_region = target_resource_region
        self.copy_authorization = copy_authorization


class CopyResult(msrest.serialization.Model):
    """Custom model copy result.

    All required parameters must be populated in order to send to Azure.

    :ivar model_id: Required. Identifier of the target model.
    :vartype model_id: str
    :ivar errors: Errors returned during the copy operation.
    :vartype errors: list[~azure.ai.formrecognizer.v2_0.models.ErrorInformation]
    """

    _validation = {
        'model_id': {'required': True},
    }

    _attribute_map = {
        'model_id': {'key': 'modelId', 'type': 'str'},
        'errors': {'key': 'errors', 'type': '[ErrorInformation]'},
    }

    def __init__(
        self,
        *,
        model_id: str,
        errors: Optional[List["_models.ErrorInformation"]] = None,
        **kwargs
    ):
        """
        :keyword model_id: Required. Identifier of the target model.
        :paramtype model_id: str
        :keyword errors: Errors returned during the copy operation.
        :paramtype errors: list[~azure.ai.formrecognizer.v2_0.models.ErrorInformation]
        """
        super(CopyResult, self).__init__(**kwargs)
        self.model_id = model_id
        self.errors = errors


class DataTable(msrest.serialization.Model):
    """Information about the extracted table contained in a page.

    All required parameters must be populated in order to send to Azure.

    :ivar rows: Required. Number of rows.
    :vartype rows: int
    :ivar columns: Required. Number of columns.
    :vartype columns: int
    :ivar cells: Required. List of cells contained in the table.
    :vartype cells: list[~azure.ai.formrecognizer.v2_0.models.DataTableCell]
    """

    _validation = {
        'rows': {'required': True, 'minimum': 1},
        'columns': {'required': True, 'minimum': 1},
        'cells': {'required': True},
    }

    _attribute_map = {
        'rows': {'key': 'rows', 'type': 'int'},
        'columns': {'key': 'columns', 'type': 'int'},
        'cells': {'key': 'cells', 'type': '[DataTableCell]'},
    }

    def __init__(
        self,
        *,
        rows: int,
        columns: int,
        cells: List["_models.DataTableCell"],
        **kwargs
    ):
        """
        :keyword rows: Required. Number of rows.
        :paramtype rows: int
        :keyword columns: Required. Number of columns.
        :paramtype columns: int
        :keyword cells: Required. List of cells contained in the table.
        :paramtype cells: list[~azure.ai.formrecognizer.v2_0.models.DataTableCell]
        """
        super(DataTable, self).__init__(**kwargs)
        self.rows = rows
        self.columns = columns
        self.cells = cells


class DataTableCell(msrest.serialization.Model):
    """Information about the extracted cell in a table.

    All required parameters must be populated in order to send to Azure.

    :ivar row_index: Required. Row index of the cell.
    :vartype row_index: int
    :ivar column_index: Required. Column index of the cell.
    :vartype column_index: int
    :ivar row_span: Number of rows spanned by this cell.
    :vartype row_span: int
    :ivar column_span: Number of columns spanned by this cell.
    :vartype column_span: int
    :ivar text: Required. Text content of the cell.
    :vartype text: str
    :ivar bounding_box: Required. Bounding box of the cell.
    :vartype bounding_box: list[float]
    :ivar confidence: Required. Confidence value.
    :vartype confidence: float
    :ivar elements: When includeTextDetails is set to true, a list of references to the text
     elements constituting this table cell.
    :vartype elements: list[str]
    :ivar is_header: Is the current cell a header cell?.
    :vartype is_header: bool
    :ivar is_footer: Is the current cell a footer cell?.
    :vartype is_footer: bool
    """

    _validation = {
        'row_index': {'required': True, 'minimum': 0},
        'column_index': {'required': True, 'minimum': 0},
        'row_span': {'minimum': 1},
        'column_span': {'minimum': 1},
        'text': {'required': True},
        'bounding_box': {'required': True, 'max_items': 8, 'min_items': 8},
        'confidence': {'required': True, 'maximum': 1, 'minimum': 0},
    }

    _attribute_map = {
        'row_index': {'key': 'rowIndex', 'type': 'int'},
        'column_index': {'key': 'columnIndex', 'type': 'int'},
        'row_span': {'key': 'rowSpan', 'type': 'int'},
        'column_span': {'key': 'columnSpan', 'type': 'int'},
        'text': {'key': 'text', 'type': 'str'},
        'bounding_box': {'key': 'boundingBox', 'type': '[float]'},
        'confidence': {'key': 'confidence', 'type': 'float'},
        'elements': {'key': 'elements', 'type': '[str]'},
        'is_header': {'key': 'isHeader', 'type': 'bool'},
        'is_footer': {'key': 'isFooter', 'type': 'bool'},
    }

    def __init__(
        self,
        *,
        row_index: int,
        column_index: int,
        text: str,
        bounding_box: List[float],
        confidence: float,
        row_span: Optional[int] = 1,
        column_span: Optional[int] = 1,
        elements: Optional[List[str]] = None,
        is_header: Optional[bool] = False,
        is_footer: Optional[bool] = False,
        **kwargs
    ):
        """
        :keyword row_index: Required. Row index of the cell.
        :paramtype row_index: int
        :keyword column_index: Required. Column index of the cell.
        :paramtype column_index: int
        :keyword row_span: Number of rows spanned by this cell.
        :paramtype row_span: int
        :keyword column_span: Number of columns spanned by this cell.
        :paramtype column_span: int
        :keyword text: Required. Text content of the cell.
        :paramtype text: str
        :keyword bounding_box: Required. Bounding box of the cell.
        :paramtype bounding_box: list[float]
        :keyword confidence: Required. Confidence value.
        :paramtype confidence: float
        :keyword elements: When includeTextDetails is set to true, a list of references to the text
         elements constituting this table cell.
        :paramtype elements: list[str]
        :keyword is_header: Is the current cell a header cell?.
        :paramtype is_header: bool
        :keyword is_footer: Is the current cell a footer cell?.
        :paramtype is_footer: bool
        """
        super(DataTableCell, self).__init__(**kwargs)
        self.row_index = row_index
        self.column_index = column_index
        self.row_span = row_span
        self.column_span = column_span
        self.text = text
        self.bounding_box = bounding_box
        self.confidence = confidence
        self.elements = elements
        self.is_header = is_header
        self.is_footer = is_footer


class DocumentResult(msrest.serialization.Model):
    """A set of extracted fields corresponding to the input document.

    All required parameters must be populated in order to send to Azure.

    :ivar doc_type: Required. Document type.
    :vartype doc_type: str
    :ivar page_range: Required. First and last page number where the document is found.
    :vartype page_range: list[int]
    :ivar fields: Required. Dictionary of named field values.
    :vartype fields: dict[str, ~azure.ai.formrecognizer.v2_0.models.FieldValue]
    """

    _validation = {
        'doc_type': {'required': True},
        'page_range': {'required': True, 'max_items': 2, 'min_items': 2},
        'fields': {'required': True},
    }

    _attribute_map = {
        'doc_type': {'key': 'docType', 'type': 'str'},
        'page_range': {'key': 'pageRange', 'type': '[int]'},
        'fields': {'key': 'fields', 'type': '{FieldValue}'},
    }

    def __init__(
        self,
        *,
        doc_type: str,
        page_range: List[int],
        fields: Dict[str, "_models.FieldValue"],
        **kwargs
    ):
        """
        :keyword doc_type: Required. Document type.
        :paramtype doc_type: str
        :keyword page_range: Required. First and last page number where the document is found.
        :paramtype page_range: list[int]
        :keyword fields: Required. Dictionary of named field values.
        :paramtype fields: dict[str, ~azure.ai.formrecognizer.v2_0.models.FieldValue]
        """
        super(DocumentResult, self).__init__(**kwargs)
        self.doc_type = doc_type
        self.page_range = page_range
        self.fields = fields


class ErrorInformation(msrest.serialization.Model):
    """ErrorInformation.

    All required parameters must be populated in order to send to Azure.

    :ivar code: Required.
    :vartype code: str
    :ivar message: Required.
    :vartype message: str
    """

    _validation = {
        'code': {'required': True},
        'message': {'required': True},
    }

    _attribute_map = {
        'code': {'key': 'code', 'type': 'str'},
        'message': {'key': 'message', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        code: str,
        message: str,
        **kwargs
    ):
        """
        :keyword code: Required.
        :paramtype code: str
        :keyword message: Required.
        :paramtype message: str
        """
        super(ErrorInformation, self).__init__(**kwargs)
        self.code = code
        self.message = message


class ErrorResponse(msrest.serialization.Model):
    """ErrorResponse.

    All required parameters must be populated in order to send to Azure.

    :ivar error: Required.
    :vartype error: ~azure.ai.formrecognizer.v2_0.models.ErrorInformation
    """

    _validation = {
        'error': {'required': True},
    }

    _attribute_map = {
        'error': {'key': 'error', 'type': 'ErrorInformation'},
    }

    def __init__(
        self,
        *,
        error: "_models.ErrorInformation",
        **kwargs
    ):
        """
        :keyword error: Required.
        :paramtype error: ~azure.ai.formrecognizer.v2_0.models.ErrorInformation
        """
        super(ErrorResponse, self).__init__(**kwargs)
        self.error = error


class FieldValue(msrest.serialization.Model):
    """Recognized field value.

    All required parameters must be populated in order to send to Azure.

    :ivar type: Required. Type of field value. Known values are: "string", "date", "time",
     "phoneNumber", "number", "integer", "array", "object".
    :vartype type: str or ~azure.ai.formrecognizer.v2_0.models.FieldValueType
    :ivar value_string: String value.
    :vartype value_string: str
    :ivar value_date: Date value.
    :vartype value_date: ~datetime.date
    :ivar value_time: Time value.
    :vartype value_time: ~datetime.time
    :ivar value_phone_number: Phone number value.
    :vartype value_phone_number: str
    :ivar value_number: Floating point value.
    :vartype value_number: float
    :ivar value_integer: Integer value.
    :vartype value_integer: int
    :ivar value_array: Array of field values.
    :vartype value_array: list[~azure.ai.formrecognizer.v2_0.models.FieldValue]
    :ivar value_object: Dictionary of named field values.
    :vartype value_object: dict[str, ~azure.ai.formrecognizer.v2_0.models.FieldValue]
    :ivar text: Text content of the extracted field.
    :vartype text: str
    :ivar bounding_box: Bounding box of the field value, if appropriate.
    :vartype bounding_box: list[float]
    :ivar confidence: Confidence score.
    :vartype confidence: float
    :ivar elements: When includeTextDetails is set to true, a list of references to the text
     elements constituting this field.
    :vartype elements: list[str]
    :ivar page: The 1-based page number in the input document.
    :vartype page: int
    """

    _validation = {
        'type': {'required': True},
        'bounding_box': {'max_items': 8, 'min_items': 8},
        'confidence': {'maximum': 1, 'minimum': 0},
        'page': {'minimum': 1},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'value_string': {'key': 'valueString', 'type': 'str'},
        'value_date': {'key': 'valueDate', 'type': 'date'},
        'value_time': {'key': 'valueTime', 'type': 'time'},
        'value_phone_number': {'key': 'valuePhoneNumber', 'type': 'str'},
        'value_number': {'key': 'valueNumber', 'type': 'float'},
        'value_integer': {'key': 'valueInteger', 'type': 'int'},
        'value_array': {'key': 'valueArray', 'type': '[FieldValue]'},
        'value_object': {'key': 'valueObject', 'type': '{FieldValue}'},
        'text': {'key': 'text', 'type': 'str'},
        'bounding_box': {'key': 'boundingBox', 'type': '[float]'},
        'confidence': {'key': 'confidence', 'type': 'float'},
        'elements': {'key': 'elements', 'type': '[str]'},
        'page': {'key': 'page', 'type': 'int'},
    }

    def __init__(
        self,
        *,
        type: Union[str, "_models.FieldValueType"],
        value_string: Optional[str] = None,
        value_date: Optional[datetime.date] = None,
        value_time: Optional[datetime.time] = None,
        value_phone_number: Optional[str] = None,
        value_number: Optional[float] = None,
        value_integer: Optional[int] = None,
        value_array: Optional[List["_models.FieldValue"]] = None,
        value_object: Optional[Dict[str, "_models.FieldValue"]] = None,
        text: Optional[str] = None,
        bounding_box: Optional[List[float]] = None,
        confidence: Optional[float] = None,
        elements: Optional[List[str]] = None,
        page: Optional[int] = None,
        **kwargs
    ):
        """
        :keyword type: Required. Type of field value. Known values are: "string", "date", "time",
         "phoneNumber", "number", "integer", "array", "object".
        :paramtype type: str or ~azure.ai.formrecognizer.v2_0.models.FieldValueType
        :keyword value_string: String value.
        :paramtype value_string: str
        :keyword value_date: Date value.
        :paramtype value_date: ~datetime.date
        :keyword value_time: Time value.
        :paramtype value_time: ~datetime.time
        :keyword value_phone_number: Phone number value.
        :paramtype value_phone_number: str
        :keyword value_number: Floating point value.
        :paramtype value_number: float
        :keyword value_integer: Integer value.
        :paramtype value_integer: int
        :keyword value_array: Array of field values.
        :paramtype value_array: list[~azure.ai.formrecognizer.v2_0.models.FieldValue]
        :keyword value_object: Dictionary of named field values.
        :paramtype value_object: dict[str, ~azure.ai.formrecognizer.v2_0.models.FieldValue]
        :keyword text: Text content of the extracted field.
        :paramtype text: str
        :keyword bounding_box: Bounding box of the field value, if appropriate.
        :paramtype bounding_box: list[float]
        :keyword confidence: Confidence score.
        :paramtype confidence: float
        :keyword elements: When includeTextDetails is set to true, a list of references to the text
         elements constituting this field.
        :paramtype elements: list[str]
        :keyword page: The 1-based page number in the input document.
        :paramtype page: int
        """
        super(FieldValue, self).__init__(**kwargs)
        self.type = type
        self.value_string = value_string
        self.value_date = value_date
        self.value_time = value_time
        self.value_phone_number = value_phone_number
        self.value_number = value_number
        self.value_integer = value_integer
        self.value_array = value_array
        self.value_object = value_object
        self.text = text
        self.bounding_box = bounding_box
        self.confidence = confidence
        self.elements = elements
        self.page = page


class FormFieldsReport(msrest.serialization.Model):
    """Report for a custom model training field.

    All required parameters must be populated in order to send to Azure.

    :ivar field_name: Required. Training field name.
    :vartype field_name: str
    :ivar accuracy: Required. Estimated extraction accuracy for this field.
    :vartype accuracy: float
    """

    _validation = {
        'field_name': {'required': True},
        'accuracy': {'required': True},
    }

    _attribute_map = {
        'field_name': {'key': 'fieldName', 'type': 'str'},
        'accuracy': {'key': 'accuracy', 'type': 'float'},
    }

    def __init__(
        self,
        *,
        field_name: str,
        accuracy: float,
        **kwargs
    ):
        """
        :keyword field_name: Required. Training field name.
        :paramtype field_name: str
        :keyword accuracy: Required. Estimated extraction accuracy for this field.
        :paramtype accuracy: float
        """
        super(FormFieldsReport, self).__init__(**kwargs)
        self.field_name = field_name
        self.accuracy = accuracy


class KeysResult(msrest.serialization.Model):
    """Keys extracted by the custom model.

    All required parameters must be populated in order to send to Azure.

    :ivar clusters: Required. Object mapping clusterIds to a list of keys.
    :vartype clusters: dict[str, list[str]]
    """

    _validation = {
        'clusters': {'required': True},
    }

    _attribute_map = {
        'clusters': {'key': 'clusters', 'type': '{[str]}'},
    }

    def __init__(
        self,
        *,
        clusters: Dict[str, List[str]],
        **kwargs
    ):
        """
        :keyword clusters: Required. Object mapping clusterIds to a list of keys.
        :paramtype clusters: dict[str, list[str]]
        """
        super(KeysResult, self).__init__(**kwargs)
        self.clusters = clusters


class KeyValueElement(msrest.serialization.Model):
    """Information about the extracted key or value in a key-value pair.

    All required parameters must be populated in order to send to Azure.

    :ivar text: Required. The text content of the key or value.
    :vartype text: str
    :ivar bounding_box: Bounding box of the key or value.
    :vartype bounding_box: list[float]
    :ivar elements: When includeTextDetails is set to true, a list of references to the text
     elements constituting this key or value.
    :vartype elements: list[str]
    """

    _validation = {
        'text': {'required': True},
        'bounding_box': {'max_items': 8, 'min_items': 8},
    }

    _attribute_map = {
        'text': {'key': 'text', 'type': 'str'},
        'bounding_box': {'key': 'boundingBox', 'type': '[float]'},
        'elements': {'key': 'elements', 'type': '[str]'},
    }

    def __init__(
        self,
        *,
        text: str,
        bounding_box: Optional[List[float]] = None,
        elements: Optional[List[str]] = None,
        **kwargs
    ):
        """
        :keyword text: Required. The text content of the key or value.
        :paramtype text: str
        :keyword bounding_box: Bounding box of the key or value.
        :paramtype bounding_box: list[float]
        :keyword elements: When includeTextDetails is set to true, a list of references to the text
         elements constituting this key or value.
        :paramtype elements: list[str]
        """
        super(KeyValueElement, self).__init__(**kwargs)
        self.text = text
        self.bounding_box = bounding_box
        self.elements = elements


class KeyValuePair(msrest.serialization.Model):
    """Information about the extracted key-value pair.

    All required parameters must be populated in order to send to Azure.

    :ivar label: A user defined label for the key/value pair entry.
    :vartype label: str
    :ivar key: Required. Information about the extracted key in a key-value pair.
    :vartype key: ~azure.ai.formrecognizer.v2_0.models.KeyValueElement
    :ivar value: Required. Information about the extracted value in a key-value pair.
    :vartype value: ~azure.ai.formrecognizer.v2_0.models.KeyValueElement
    :ivar confidence: Required. Confidence value.
    :vartype confidence: float
    """

    _validation = {
        'key': {'required': True},
        'value': {'required': True},
        'confidence': {'required': True, 'maximum': 1, 'minimum': 0},
    }

    _attribute_map = {
        'label': {'key': 'label', 'type': 'str'},
        'key': {'key': 'key', 'type': 'KeyValueElement'},
        'value': {'key': 'value', 'type': 'KeyValueElement'},
        'confidence': {'key': 'confidence', 'type': 'float'},
    }

    def __init__(
        self,
        *,
        key: "_models.KeyValueElement",
        value: "_models.KeyValueElement",
        confidence: float,
        label: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword label: A user defined label for the key/value pair entry.
        :paramtype label: str
        :keyword key: Required. Information about the extracted key in a key-value pair.
        :paramtype key: ~azure.ai.formrecognizer.v2_0.models.KeyValueElement
        :keyword value: Required. Information about the extracted value in a key-value pair.
        :paramtype value: ~azure.ai.formrecognizer.v2_0.models.KeyValueElement
        :keyword confidence: Required. Confidence value.
        :paramtype confidence: float
        """
        super(KeyValuePair, self).__init__(**kwargs)
        self.label = label
        self.key = key
        self.value = value
        self.confidence = confidence


class Model(msrest.serialization.Model):
    """Response to the get custom model operation.

    All required parameters must be populated in order to send to Azure.

    :ivar model_info: Required. Basic custom model information.
    :vartype model_info: ~azure.ai.formrecognizer.v2_0.models.ModelInfo
    :ivar keys: Keys extracted by the custom model.
    :vartype keys: ~azure.ai.formrecognizer.v2_0.models.KeysResult
    :ivar train_result: Custom model training result.
    :vartype train_result: ~azure.ai.formrecognizer.v2_0.models.TrainResult
    """

    _validation = {
        'model_info': {'required': True},
    }

    _attribute_map = {
        'model_info': {'key': 'modelInfo', 'type': 'ModelInfo'},
        'keys': {'key': 'keys', 'type': 'KeysResult'},
        'train_result': {'key': 'trainResult', 'type': 'TrainResult'},
    }

    def __init__(
        self,
        *,
        model_info: "_models.ModelInfo",
        keys: Optional["_models.KeysResult"] = None,
        train_result: Optional["_models.TrainResult"] = None,
        **kwargs
    ):
        """
        :keyword model_info: Required. Basic custom model information.
        :paramtype model_info: ~azure.ai.formrecognizer.v2_0.models.ModelInfo
        :keyword keys: Keys extracted by the custom model.
        :paramtype keys: ~azure.ai.formrecognizer.v2_0.models.KeysResult
        :keyword train_result: Custom model training result.
        :paramtype train_result: ~azure.ai.formrecognizer.v2_0.models.TrainResult
        """
        super(Model, self).__init__(**kwargs)
        self.model_info = model_info
        self.keys = keys
        self.train_result = train_result


class ModelInfo(msrest.serialization.Model):
    """Basic custom model information.

    All required parameters must be populated in order to send to Azure.

    :ivar model_id: Required. Model identifier.
    :vartype model_id: str
    :ivar status: Required. Status of the model. Known values are: "creating", "ready", "invalid".
    :vartype status: str or ~azure.ai.formrecognizer.v2_0.models.ModelStatus
    :ivar created_date_time: Required. Date and time (UTC) when the model was created.
    :vartype created_date_time: ~datetime.datetime
    :ivar last_updated_date_time: Required. Date and time (UTC) when the status was last updated.
    :vartype last_updated_date_time: ~datetime.datetime
    """

    _validation = {
        'model_id': {'required': True},
        'status': {'required': True},
        'created_date_time': {'required': True},
        'last_updated_date_time': {'required': True},
    }

    _attribute_map = {
        'model_id': {'key': 'modelId', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_updated_date_time': {'key': 'lastUpdatedDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        *,
        model_id: str,
        status: Union[str, "_models.ModelStatus"],
        created_date_time: datetime.datetime,
        last_updated_date_time: datetime.datetime,
        **kwargs
    ):
        """
        :keyword model_id: Required. Model identifier.
        :paramtype model_id: str
        :keyword status: Required. Status of the model. Known values are: "creating", "ready",
         "invalid".
        :paramtype status: str or ~azure.ai.formrecognizer.v2_0.models.ModelStatus
        :keyword created_date_time: Required. Date and time (UTC) when the model was created.
        :paramtype created_date_time: ~datetime.datetime
        :keyword last_updated_date_time: Required. Date and time (UTC) when the status was last
         updated.
        :paramtype last_updated_date_time: ~datetime.datetime
        """
        super(ModelInfo, self).__init__(**kwargs)
        self.model_id = model_id
        self.status = status
        self.created_date_time = created_date_time
        self.last_updated_date_time = last_updated_date_time


class Models(msrest.serialization.Model):
    """Response to the list custom models operation.

    :ivar summary: Summary of all trained custom models.
    :vartype summary: ~azure.ai.formrecognizer.v2_0.models.ModelsSummary
    :ivar model_list: Collection of trained custom models.
    :vartype model_list: list[~azure.ai.formrecognizer.v2_0.models.ModelInfo]
    :ivar next_link: Link to the next page of custom models.
    :vartype next_link: str
    """

    _attribute_map = {
        'summary': {'key': 'summary', 'type': 'ModelsSummary'},
        'model_list': {'key': 'modelList', 'type': '[ModelInfo]'},
        'next_link': {'key': 'nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        summary: Optional["_models.ModelsSummary"] = None,
        model_list: Optional[List["_models.ModelInfo"]] = None,
        next_link: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword summary: Summary of all trained custom models.
        :paramtype summary: ~azure.ai.formrecognizer.v2_0.models.ModelsSummary
        :keyword model_list: Collection of trained custom models.
        :paramtype model_list: list[~azure.ai.formrecognizer.v2_0.models.ModelInfo]
        :keyword next_link: Link to the next page of custom models.
        :paramtype next_link: str
        """
        super(Models, self).__init__(**kwargs)
        self.summary = summary
        self.model_list = model_list
        self.next_link = next_link


class ModelsSummary(msrest.serialization.Model):
    """Summary of all trained custom models.

    All required parameters must be populated in order to send to Azure.

    :ivar count: Required. Current count of trained custom models.
    :vartype count: int
    :ivar limit: Required. Max number of models that can be trained for this account.
    :vartype limit: int
    :ivar last_updated_date_time: Required. Date and time (UTC) when the summary was last updated.
    :vartype last_updated_date_time: ~datetime.datetime
    """

    _validation = {
        'count': {'required': True},
        'limit': {'required': True},
        'last_updated_date_time': {'required': True},
    }

    _attribute_map = {
        'count': {'key': 'count', 'type': 'int'},
        'limit': {'key': 'limit', 'type': 'int'},
        'last_updated_date_time': {'key': 'lastUpdatedDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        *,
        count: int,
        limit: int,
        last_updated_date_time: datetime.datetime,
        **kwargs
    ):
        """
        :keyword count: Required. Current count of trained custom models.
        :paramtype count: int
        :keyword limit: Required. Max number of models that can be trained for this account.
        :paramtype limit: int
        :keyword last_updated_date_time: Required. Date and time (UTC) when the summary was last
         updated.
        :paramtype last_updated_date_time: ~datetime.datetime
        """
        super(ModelsSummary, self).__init__(**kwargs)
        self.count = count
        self.limit = limit
        self.last_updated_date_time = last_updated_date_time


class PageResult(msrest.serialization.Model):
    """Extracted information from a single page.

    All required parameters must be populated in order to send to Azure.

    :ivar page: Required. Page number.
    :vartype page: int
    :ivar cluster_id: Cluster identifier.
    :vartype cluster_id: int
    :ivar key_value_pairs: List of key-value pairs extracted from the page.
    :vartype key_value_pairs: list[~azure.ai.formrecognizer.v2_0.models.KeyValuePair]
    :ivar tables: List of data tables extracted from the page.
    :vartype tables: list[~azure.ai.formrecognizer.v2_0.models.DataTable]
    """

    _validation = {
        'page': {'required': True, 'minimum': 1},
        'cluster_id': {'minimum': 0},
    }

    _attribute_map = {
        'page': {'key': 'page', 'type': 'int'},
        'cluster_id': {'key': 'clusterId', 'type': 'int'},
        'key_value_pairs': {'key': 'keyValuePairs', 'type': '[KeyValuePair]'},
        'tables': {'key': 'tables', 'type': '[DataTable]'},
    }

    def __init__(
        self,
        *,
        page: int,
        cluster_id: Optional[int] = None,
        key_value_pairs: Optional[List["_models.KeyValuePair"]] = None,
        tables: Optional[List["_models.DataTable"]] = None,
        **kwargs
    ):
        """
        :keyword page: Required. Page number.
        :paramtype page: int
        :keyword cluster_id: Cluster identifier.
        :paramtype cluster_id: int
        :keyword key_value_pairs: List of key-value pairs extracted from the page.
        :paramtype key_value_pairs: list[~azure.ai.formrecognizer.v2_0.models.KeyValuePair]
        :keyword tables: List of data tables extracted from the page.
        :paramtype tables: list[~azure.ai.formrecognizer.v2_0.models.DataTable]
        """
        super(PageResult, self).__init__(**kwargs)
        self.page = page
        self.cluster_id = cluster_id
        self.key_value_pairs = key_value_pairs
        self.tables = tables


class ReadResult(msrest.serialization.Model):
    """Text extracted from a page in the input document.

    All required parameters must be populated in order to send to Azure.

    :ivar page: Required. The 1-based page number in the input document.
    :vartype page: int
    :ivar angle: Required. The general orientation of the text in clockwise direction, measured in
     degrees between (-180, 180].
    :vartype angle: float
    :ivar width: Required. The width of the image/PDF in pixels/inches, respectively.
    :vartype width: float
    :ivar height: Required. The height of the image/PDF in pixels/inches, respectively.
    :vartype height: float
    :ivar unit: Required. The unit used by the width, height and boundingBox properties. For
     images, the unit is "pixel". For PDF, the unit is "inch". Known values are: "pixel", "inch".
    :vartype unit: str or ~azure.ai.formrecognizer.v2_0.models.LengthUnit
    :ivar language: The detected language on the page overall. Known values are: "en", "es".
    :vartype language: str or ~azure.ai.formrecognizer.v2_0.models.Language
    :ivar lines: When includeTextDetails is set to true, a list of recognized text lines. The
     maximum number of lines returned is 300 per page. The lines are sorted top to bottom, left to
     right, although in certain cases proximity is treated with higher priority. As the sorting
     order depends on the detected text, it may change across images and OCR version updates. Thus,
     business logic should be built upon the actual line location instead of order.
    :vartype lines: list[~azure.ai.formrecognizer.v2_0.models.TextLine]
    """

    _validation = {
        'page': {'required': True, 'minimum': 1},
        'angle': {'required': True, 'maximum': 180, 'minimum_ex': -180},
        'width': {'required': True, 'minimum': 0},
        'height': {'required': True, 'minimum': 0},
        'unit': {'required': True},
    }

    _attribute_map = {
        'page': {'key': 'page', 'type': 'int'},
        'angle': {'key': 'angle', 'type': 'float'},
        'width': {'key': 'width', 'type': 'float'},
        'height': {'key': 'height', 'type': 'float'},
        'unit': {'key': 'unit', 'type': 'str'},
        'language': {'key': 'language', 'type': 'str'},
        'lines': {'key': 'lines', 'type': '[TextLine]'},
    }

    def __init__(
        self,
        *,
        page: int,
        angle: float,
        width: float,
        height: float,
        unit: Union[str, "_models.LengthUnit"],
        language: Optional[Union[str, "_models.Language"]] = None,
        lines: Optional[List["_models.TextLine"]] = None,
        **kwargs
    ):
        """
        :keyword page: Required. The 1-based page number in the input document.
        :paramtype page: int
        :keyword angle: Required. The general orientation of the text in clockwise direction, measured
         in degrees between (-180, 180].
        :paramtype angle: float
        :keyword width: Required. The width of the image/PDF in pixels/inches, respectively.
        :paramtype width: float
        :keyword height: Required. The height of the image/PDF in pixels/inches, respectively.
        :paramtype height: float
        :keyword unit: Required. The unit used by the width, height and boundingBox properties. For
         images, the unit is "pixel". For PDF, the unit is "inch". Known values are: "pixel", "inch".
        :paramtype unit: str or ~azure.ai.formrecognizer.v2_0.models.LengthUnit
        :keyword language: The detected language on the page overall. Known values are: "en", "es".
        :paramtype language: str or ~azure.ai.formrecognizer.v2_0.models.Language
        :keyword lines: When includeTextDetails is set to true, a list of recognized text lines. The
         maximum number of lines returned is 300 per page. The lines are sorted top to bottom, left to
         right, although in certain cases proximity is treated with higher priority. As the sorting
         order depends on the detected text, it may change across images and OCR version updates. Thus,
         business logic should be built upon the actual line location instead of order.
        :paramtype lines: list[~azure.ai.formrecognizer.v2_0.models.TextLine]
        """
        super(ReadResult, self).__init__(**kwargs)
        self.page = page
        self.angle = angle
        self.width = width
        self.height = height
        self.unit = unit
        self.language = language
        self.lines = lines


class SourcePath(msrest.serialization.Model):
    """Uri or local path to source data.

    :ivar source: File source path.
    :vartype source: str
    """

    _validation = {
        'source': {'max_length': 2048, 'min_length': 0},
    }

    _attribute_map = {
        'source': {'key': 'source', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        source: Optional[str] = None,
        **kwargs
    ):
        """
        :keyword source: File source path.
        :paramtype source: str
        """
        super(SourcePath, self).__init__(**kwargs)
        self.source = source


class TextLine(msrest.serialization.Model):
    """An object representing an extracted text line.

    All required parameters must be populated in order to send to Azure.

    :ivar text: Required. The text content of the line.
    :vartype text: str
    :ivar bounding_box: Required. Bounding box of an extracted line.
    :vartype bounding_box: list[float]
    :ivar language: The detected language of this line, if different from the overall page
     language. Known values are: "en", "es".
    :vartype language: str or ~azure.ai.formrecognizer.v2_0.models.Language
    :ivar words: Required. List of words in the text line.
    :vartype words: list[~azure.ai.formrecognizer.v2_0.models.TextWord]
    """

    _validation = {
        'text': {'required': True},
        'bounding_box': {'required': True, 'max_items': 8, 'min_items': 8},
        'words': {'required': True},
    }

    _attribute_map = {
        'text': {'key': 'text', 'type': 'str'},
        'bounding_box': {'key': 'boundingBox', 'type': '[float]'},
        'language': {'key': 'language', 'type': 'str'},
        'words': {'key': 'words', 'type': '[TextWord]'},
    }

    def __init__(
        self,
        *,
        text: str,
        bounding_box: List[float],
        words: List["_models.TextWord"],
        language: Optional[Union[str, "_models.Language"]] = None,
        **kwargs
    ):
        """
        :keyword text: Required. The text content of the line.
        :paramtype text: str
        :keyword bounding_box: Required. Bounding box of an extracted line.
        :paramtype bounding_box: list[float]
        :keyword language: The detected language of this line, if different from the overall page
         language. Known values are: "en", "es".
        :paramtype language: str or ~azure.ai.formrecognizer.v2_0.models.Language
        :keyword words: Required. List of words in the text line.
        :paramtype words: list[~azure.ai.formrecognizer.v2_0.models.TextWord]
        """
        super(TextLine, self).__init__(**kwargs)
        self.text = text
        self.bounding_box = bounding_box
        self.language = language
        self.words = words


class TextWord(msrest.serialization.Model):
    """An object representing a word.

    All required parameters must be populated in order to send to Azure.

    :ivar text: Required. The text content of the word.
    :vartype text: str
    :ivar bounding_box: Required. Bounding box of an extracted word.
    :vartype bounding_box: list[float]
    :ivar confidence: Confidence value.
    :vartype confidence: float
    """

    _validation = {
        'text': {'required': True},
        'bounding_box': {'required': True, 'max_items': 8, 'min_items': 8},
        'confidence': {'maximum': 1, 'minimum': 0},
    }

    _attribute_map = {
        'text': {'key': 'text', 'type': 'str'},
        'bounding_box': {'key': 'boundingBox', 'type': '[float]'},
        'confidence': {'key': 'confidence', 'type': 'float'},
    }

    def __init__(
        self,
        *,
        text: str,
        bounding_box: List[float],
        confidence: Optional[float] = None,
        **kwargs
    ):
        """
        :keyword text: Required. The text content of the word.
        :paramtype text: str
        :keyword bounding_box: Required. Bounding box of an extracted word.
        :paramtype bounding_box: list[float]
        :keyword confidence: Confidence value.
        :paramtype confidence: float
        """
        super(TextWord, self).__init__(**kwargs)
        self.text = text
        self.bounding_box = bounding_box
        self.confidence = confidence


class TrainingDocumentInfo(msrest.serialization.Model):
    """Report for a custom model training document.

    All required parameters must be populated in order to send to Azure.

    :ivar document_name: Required. Training document name.
    :vartype document_name: str
    :ivar pages: Required. Total number of pages trained.
    :vartype pages: int
    :ivar errors: Required. List of errors.
    :vartype errors: list[~azure.ai.formrecognizer.v2_0.models.ErrorInformation]
    :ivar status: Required. Status of the training operation. Known values are: "succeeded",
     "partiallySucceeded", "failed".
    :vartype status: str or ~azure.ai.formrecognizer.v2_0.models.TrainStatus
    """

    _validation = {
        'document_name': {'required': True},
        'pages': {'required': True},
        'errors': {'required': True},
        'status': {'required': True},
    }

    _attribute_map = {
        'document_name': {'key': 'documentName', 'type': 'str'},
        'pages': {'key': 'pages', 'type': 'int'},
        'errors': {'key': 'errors', 'type': '[ErrorInformation]'},
        'status': {'key': 'status', 'type': 'str'},
    }

    def __init__(
        self,
        *,
        document_name: str,
        pages: int,
        errors: List["_models.ErrorInformation"],
        status: Union[str, "_models.TrainStatus"],
        **kwargs
    ):
        """
        :keyword document_name: Required. Training document name.
        :paramtype document_name: str
        :keyword pages: Required. Total number of pages trained.
        :paramtype pages: int
        :keyword errors: Required. List of errors.
        :paramtype errors: list[~azure.ai.formrecognizer.v2_0.models.ErrorInformation]
        :keyword status: Required. Status of the training operation. Known values are: "succeeded",
         "partiallySucceeded", "failed".
        :paramtype status: str or ~azure.ai.formrecognizer.v2_0.models.TrainStatus
        """
        super(TrainingDocumentInfo, self).__init__(**kwargs)
        self.document_name = document_name
        self.pages = pages
        self.errors = errors
        self.status = status


class TrainRequest(msrest.serialization.Model):
    """Request parameter to train a new custom model.

    All required parameters must be populated in order to send to Azure.

    :ivar source: Required. Source path containing the training documents.
    :vartype source: str
    :ivar source_filter: Filter to apply to the documents in the source path for training.
    :vartype source_filter: ~azure.ai.formrecognizer.v2_0.models.TrainSourceFilter
    :ivar use_label_file: Use label file for training a model.
    :vartype use_label_file: bool
    """

    _validation = {
        'source': {'required': True, 'max_length': 2048, 'min_length': 0},
    }

    _attribute_map = {
        'source': {'key': 'source', 'type': 'str'},
        'source_filter': {'key': 'sourceFilter', 'type': 'TrainSourceFilter'},
        'use_label_file': {'key': 'useLabelFile', 'type': 'bool'},
    }

    def __init__(
        self,
        *,
        source: str,
        source_filter: Optional["_models.TrainSourceFilter"] = None,
        use_label_file: Optional[bool] = False,
        **kwargs
    ):
        """
        :keyword source: Required. Source path containing the training documents.
        :paramtype source: str
        :keyword source_filter: Filter to apply to the documents in the source path for training.
        :paramtype source_filter: ~azure.ai.formrecognizer.v2_0.models.TrainSourceFilter
        :keyword use_label_file: Use label file for training a model.
        :paramtype use_label_file: bool
        """
        super(TrainRequest, self).__init__(**kwargs)
        self.source = source
        self.source_filter = source_filter
        self.use_label_file = use_label_file


class TrainResult(msrest.serialization.Model):
    """Custom model training result.

    All required parameters must be populated in order to send to Azure.

    :ivar training_documents: Required. List of the documents used to train the model and any
     errors reported in each document.
    :vartype training_documents: list[~azure.ai.formrecognizer.v2_0.models.TrainingDocumentInfo]
    :ivar fields: List of fields used to train the model and the train operation error reported by
     each.
    :vartype fields: list[~azure.ai.formrecognizer.v2_0.models.FormFieldsReport]
    :ivar average_model_accuracy: Average accuracy.
    :vartype average_model_accuracy: float
    :ivar errors: Errors returned during the training operation.
    :vartype errors: list[~azure.ai.formrecognizer.v2_0.models.ErrorInformation]
    """

    _validation = {
        'training_documents': {'required': True},
    }

    _attribute_map = {
        'training_documents': {'key': 'trainingDocuments', 'type': '[TrainingDocumentInfo]'},
        'fields': {'key': 'fields', 'type': '[FormFieldsReport]'},
        'average_model_accuracy': {'key': 'averageModelAccuracy', 'type': 'float'},
        'errors': {'key': 'errors', 'type': '[ErrorInformation]'},
    }

    def __init__(
        self,
        *,
        training_documents: List["_models.TrainingDocumentInfo"],
        fields: Optional[List["_models.FormFieldsReport"]] = None,
        average_model_accuracy: Optional[float] = None,
        errors: Optional[List["_models.ErrorInformation"]] = None,
        **kwargs
    ):
        """
        :keyword training_documents: Required. List of the documents used to train the model and any
         errors reported in each document.
        :paramtype training_documents: list[~azure.ai.formrecognizer.v2_0.models.TrainingDocumentInfo]
        :keyword fields: List of fields used to train the model and the train operation error reported
         by each.
        :paramtype fields: list[~azure.ai.formrecognizer.v2_0.models.FormFieldsReport]
        :keyword average_model_accuracy: Average accuracy.
        :paramtype average_model_accuracy: float
        :keyword errors: Errors returned during the training operation.
        :paramtype errors: list[~azure.ai.formrecognizer.v2_0.models.ErrorInformation]
        """
        super(TrainResult, self).__init__(**kwargs)
        self.training_documents = training_documents
        self.fields = fields
        self.average_model_accuracy = average_model_accuracy
        self.errors = errors


class TrainSourceFilter(msrest.serialization.Model):
    """Filter to apply to the documents in the source path for training.

    :ivar prefix: A case-sensitive prefix string to filter documents in the source path for
     training. For example, when using a Azure storage blob Uri, use the prefix to restrict sub
     folders for training.
    :vartype prefix: str
    :ivar include_sub_folders: A flag to indicate if sub folders within the set of prefix folders
     will also need to be included when searching for content to be preprocessed.
    :vartype include_sub_folders: bool
    """

    _validation = {
        'prefix': {'max_length': 1024, 'min_length': 0},
    }

    _attribute_map = {
        'prefix': {'key': 'prefix', 'type': 'str'},
        'include_sub_folders': {'key': 'includeSubFolders', 'type': 'bool'},
    }

    def __init__(
        self,
        *,
        prefix: Optional[str] = None,
        include_sub_folders: Optional[bool] = False,
        **kwargs
    ):
        """
        :keyword prefix: A case-sensitive prefix string to filter documents in the source path for
         training. For example, when using a Azure storage blob Uri, use the prefix to restrict sub
         folders for training.
        :paramtype prefix: str
        :keyword include_sub_folders: A flag to indicate if sub folders within the set of prefix
         folders will also need to be included when searching for content to be preprocessed.
        :paramtype include_sub_folders: bool
        """
        super(TrainSourceFilter, self).__init__(**kwargs)
        self.prefix = prefix
        self.include_sub_folders = include_sub_folders



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: sample_classify_document_from_url_async.py

DESCRIPTION:
    This sample demonstrates how to classify a document from a URL using a trained document classifier.
    To learn how to build your custom classifier, see sample_build_classifier.py.

    More details on building a classifier and labeling your data can be found here:
    https://aka.ms/azsdk/formrecognizer/buildclassifiermodel

USAGE:
    python sample_classify_document_from_url_async.py

    Set the environment variables with your own values before running the sample:
    1) AZURE_FORM_RECOGNIZER_ENDPOINT - the endpoint to your Form Recognizer resource.
    2) AZURE_FORM_RECOGNIZER_KEY - your Form Recognizer API key
    3) CLASSIFIER_ID - the ID of your trained document classifier
        -OR-
       CLASSIFIER_CONTAINER_SAS_URL - The shared access signature (SAS) Url of your Azure Blob Storage container with your training files.
       A document classifier will be built and used to run the sample.
"""

import os
import asyncio


async def classify_document_from_url_async(classifier_id):
    # [START classify_document_from_url_async]
    from azure.core.credentials import AzureKeyCredential
    from azure.ai.formrecognizer.aio import DocumentAnalysisClient

    endpoint = os.environ["AZURE_FORM_RECOGNIZER_ENDPOINT"]
    key = os.environ["AZURE_FORM_RECOGNIZER_KEY"]
    classifier_id = os.getenv("CLASSIFIER_ID", classifier_id)

    document_analysis_client = DocumentAnalysisClient(
        endpoint=endpoint, credential=AzureKeyCredential(key)
    )
    async with document_analysis_client:
        url = "https://raw.githubusercontent.com/Azure/azure-sdk-for-python/main/sdk/formrecognizer/azure-ai-formrecognizer/tests/sample_forms/forms/IRS-1040.pdf"

        poller = await document_analysis_client.begin_classify_document_from_url(
            classifier_id, document_url=url
        )
        result = await poller.result()

    print("----Classified documents----")
    for doc in result.documents:
        print(
            f"Found document of type '{doc.doc_type or 'N/A'}' with a confidence of {doc.confidence} contained on "
            f"the following pages: {[region.page_number for region in doc.bounding_regions]}"
        )
    # [END classify_document_from_url_async]


async def main():
    classifier_id = None
    if os.getenv("CLASSIFIER_CONTAINER_SAS_URL") and not os.getenv("CLASSIFIER_ID"):
        from azure.core.credentials import AzureKeyCredential
        from azure.ai.formrecognizer.aio import DocumentModelAdministrationClient
        from azure.ai.formrecognizer import (
            ClassifierDocumentTypeDetails,
            BlobSource,
        )

        endpoint = os.environ["AZURE_FORM_RECOGNIZER_ENDPOINT"]
        key = os.environ["AZURE_FORM_RECOGNIZER_KEY"]
        blob_container_sas_url = os.environ["CLASSIFIER_CONTAINER_SAS_URL"]

        document_model_admin_client = DocumentModelAdministrationClient(
            endpoint=endpoint, credential=AzureKeyCredential(key)
        )
        async with document_model_admin_client:
            poller = await document_model_admin_client.begin_build_document_classifier(
                doc_types={
                    "IRS-1040-A": ClassifierDocumentTypeDetails(
                        source=BlobSource(
                            container_url=blob_container_sas_url,
                            prefix="IRS-1040-A/train",
                        )
                    ),
                    "IRS-1040-D": ClassifierDocumentTypeDetails(
                        source=BlobSource(
                            container_url=blob_container_sas_url,
                            prefix="IRS-1040-D/train",
                        )
                    ),
                },
            )
            classifier = await poller.result()
            classifier_id = classifier.classifier_id

    await classify_document_from_url_async(classifier_id)


if __name__ == "__main__":
    from azure.core.exceptions import HttpResponseError

    try:
        asyncio.run(main())
    except HttpResponseError as error:
        print(
            "For more information about troubleshooting errors, see the following guide: "
            "https://aka.ms/azsdk/python/formrecognizer/troubleshooting"
        )
        # Examples of how to check an HttpResponseError
        # Check by error code:
        if error.error is not None:
            if error.error.code == "InvalidImage":
                print(f"Received an invalid image error: {error.error}")
            if error.error.code == "InvalidRequest":
                print(f"Received an invalid request error: {error.error}")
            # Raise the error again after printing it
            raise
        # If the inner error is None and then it is possible to check the message to get more information:
        if "Invalid request".casefold() in error.message.casefold():
            print(f"Uh-oh! Seems there was an invalid request: {error}")
        # Raise the error again
        raise



# coding: utf-8

# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

"""
FILE: sample_get_operations_async.py

DESCRIPTION:
    This sample demonstrates how to list/get all document model operations (succeeded, in-progress, failed)
    associated with the Form Recognizer resource. Kinds of operations returned are "documentModelBuild",
    "documentModelCompose", and "documentModelCopyTo". Note that operation information only persists for
    24 hours. If the operation was successful, the document model can be accessed using get_document_model or list_document_models APIs.

USAGE:
    python sample_get_operations_async.py

    Set the environment variables with your own values before running the sample:
    1) AZURE_FORM_RECOGNIZER_ENDPOINT - the endpoint to your Form Recognizer resource.
    2) AZURE_FORM_RECOGNIZER_KEY - your Form Recognizer API key
"""

import os
import asyncio


async def sample_get_operations_async():
    # [START list_operations_async]
    from azure.core.credentials import AzureKeyCredential
    from azure.ai.formrecognizer.aio import DocumentModelAdministrationClient

    endpoint = os.environ["AZURE_FORM_RECOGNIZER_ENDPOINT"]
    key = os.environ["AZURE_FORM_RECOGNIZER_KEY"]

    document_model_admin_client = DocumentModelAdministrationClient(
        endpoint=endpoint, credential=AzureKeyCredential(key)
    )

    async with document_model_admin_client:
        operations = document_model_admin_client.list_operations()

        print("The following document model operations exist under my resource:")
        async for operation in operations:
            print(f"\nOperation ID: {operation.operation_id}")
            print(f"Operation kind: {operation.kind}")
            print(f"Operation status: {operation.status}")
            print(f"Operation percent completed: {operation.percent_completed}")
            print(f"Operation created on: {operation.created_on}")
            print(f"Operation last updated on: {operation.last_updated_on}")
            print(
                f"Resource location of successful operation: {operation.resource_location}"
            )
        # [END list_operations_async]

        # [START get_operation_async]
        # Get an operation by ID
        try:
            first_operation = await operations.__anext__()

            print(f"\nGetting operation info by ID: {first_operation.operation_id}")
            operation_info = await document_model_admin_client.get_operation(
                first_operation.operation_id
            )
            if operation_info.status == "succeeded":
                print(f"My {operation_info.kind} operation is completed.")
                result = operation_info.result
                if result is not None:
                    if operation_info.kind == "documentClassifierBuild":
                        print(f"Classifier ID: {result.classifier_id}")
                    else:
                        print(f"Model ID: {result.model_id}")
            elif operation_info.status == "failed":
                print(f"My {operation_info.kind} operation failed.")
                error = operation_info.error
                if error is not None:
                    print(f"{error.code}: {error.message}")
            else:
                print(f"My operation status is {operation_info.status}")
        except StopAsyncIteration:
            print("No operations found.")
        # [END get_operation_async]


async def main():
    await sample_get_operations_async()


if __name__ == "__main__":
    import sys
    from azure.core.exceptions import HttpResponseError

    try:
        asyncio.run(main())
    except HttpResponseError as error:
        print(
            "For more information about troubleshooting errors, see the following guide: "
            "https://aka.ms/azsdk/python/formrecognizer/troubleshooting"
        )
        # Examples of how to check an HttpResponseError
        # Check by error code:
        if error.error is not None:
            if error.error.code == "InvalidImage":
                print(f"Received an invalid image error: {error.error}")
            if error.error.code == "InvalidRequest":
                print(f"Received an invalid request error: {error.error}")
            # Raise the error again after printing it
            raise
        # If the inner error is None and then it is possible to check the message to get more information:
        if "Invalid request".casefold() in error.message.casefold():
            print(f"Uh-oh! Seems there was an invalid request: {error}")
        # Raise the error again
        raise



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

# pylint: disable=protected-access

import datetime
import json
from typing import Callable, Mapping, TypeVar, Any, Optional
from typing_extensions import Protocol, runtime_checkable
from azure.core.exceptions import HttpResponseError, ODataV4Format
from azure.core.pipeline import PipelineResponse
from azure.core.polling import LROPoller, PollingMethod
from azure.core.polling.base_polling import (
    LocationPolling,
    OperationResourcePolling,
    _is_empty,
    _as_json,
    BadResponse,
)

PollingReturnType_co = TypeVar("PollingReturnType_co", covariant=True)


def raise_error(response, errors, message):
    error_message = "({}) {}{}".format(errors[0]["code"], errors[0]["message"], message)
    error = HttpResponseError(message=error_message, response=response)
    error.error = ODataV4Format(errors[0])
    raise error


def parse_operation_id(location):
    prefix = location.split("?api-version")[0]
    operation_id = prefix.split("/operations/")[1]
    return operation_id


@runtime_checkable
class DocumentModelAdministrationLROPoller(Protocol[PollingReturnType_co]):
    """Implements a protocol followed by returned poller objects."""

    @property
    def details(  # pylint: disable=unused-argument
        self,
    ) -> Mapping[str, Any]:
        ...

    def polling_method(
        self,
    ) -> PollingMethod[PollingReturnType_co]:
        ...

    def continuation_token(self) -> str:
        ...

    def status(self) -> str:
        ...

    def result(  # pylint: disable=unused-argument
        self, timeout: Optional[int] = None
    ) -> PollingReturnType_co:
        ...

    def wait(self, timeout: Optional[float] = None) -> None:  # pylint: disable=unused-argument
        ...

    def done(self) -> bool:
        ...

    def add_done_callback(self, func: Callable) -> None:  # pylint: disable=unused-argument
        ...

    def remove_done_callback(self, func: Callable) -> None:  # pylint: disable=unused-argument
        ...


class DocumentModelAdministrationClientLROPoller(LROPoller[PollingReturnType_co]):  # pylint: disable=name-too-long
    """Custom poller for model build operations. Call `result()` on the poller to return
    a :class:`~azure.ai.formrecognizer.DocumentModelDetails`.

    .. versionadded:: 2022-08-31
        The *DocumentModelAdministrationLROPoller* poller object
    """

    @property
    def _current_body(self):
        body = self.polling_method()._pipeline_response.http_response.text()
        if body:
            return json.loads(body)
        return {}

    @property
    def details(self) -> Mapping[str, Any]:
        """Returns metadata associated with the long-running operation.

        :return: Returns metadata associated with the long-running operation.
        :rtype: Mapping[str, Any]
        """
        created_on = self._current_body.get("createdDateTime", None)
        if created_on:
            created_on = datetime.datetime.strptime(created_on, "%Y-%m-%dT%H:%M:%SZ")
        last_updated_on = self._current_body.get("lastUpdatedDateTime", None)
        if last_updated_on:
            last_updated_on = datetime.datetime.strptime(last_updated_on, "%Y-%m-%dT%H:%M:%SZ")
        return {
            "operation_id": parse_operation_id(
                self.polling_method()._initial_response.http_response.headers["Operation-Location"]  # type: ignore
            ),
            "operation_kind": self._current_body.get("kind", None),
            "percent_completed": self._current_body.get("percentCompleted", 0),
            "resource_location_url": self._current_body.get("resourceLocation", None),
            "created_on": created_on,
            "last_updated_on": last_updated_on,
        }

    @classmethod
    def from_continuation_token(
        cls, polling_method: PollingMethod[PollingReturnType_co], continuation_token: str, **kwargs: Any
    ) -> "DocumentModelAdministrationClientLROPoller":
        (
            client,
            initial_response,
            deserialization_callback,
        ) = polling_method.from_continuation_token(continuation_token, **kwargs)

        return cls(client, initial_response, deserialization_callback, polling_method)


class DocumentModelAdministrationPolling(OperationResourcePolling):
    """Polling method overrides for training endpoints."""

    def get_final_get_url(self, pipeline_response: Any) -> None:
        """If a final GET is needed, returns the URL.

        :param any pipeline_response: The pipeline response to get the final url.
        :rtype: None
        """
        return None


class FormTrainingPolling(LocationPolling):
    """Polling method overrides for training endpoints."""

    def get_polling_url(self) -> str:
        """Return the polling URL.

        :return: Returns the polling URL.
        :rtype: str
        """
        return self._location_url + "?includeKeys=true"

    def get_status(self, pipeline_response: PipelineResponse) -> str:
        """Process the latest status update retrieved from a 'location' header.

        :param azure.core.pipeline.PipelineResponse pipeline_response: latest REST call response.
        :return: Returns the latest status from the 'location' header.
        :rtype: str
        :raises: BadResponse if response has no body.
        """
        response = pipeline_response.http_response
        if response.status_code == 200:
            body = _as_json(response)
            status = body["modelInfo"]["status"]
            if not status:
                raise BadResponse("No status found in body")
            if status.lower() == "invalid":
                train_result = body.get("trainResult")
                if train_result:
                    errors = train_result.get("errors")
                    if errors:
                        message = "\nInvalid model created with ID={}".format(body["modelInfo"]["modelId"])
                        raise_error(response, errors, message)
                return "Failed"
            if status.lower() != "creating":
                return "Succeeded"

            return "InProgress"

        return "Failed"


class AnalyzePolling(OperationResourcePolling):
    """Polling method overrides for custom analyze endpoints."""

    def get_status(self, pipeline_response: PipelineResponse) -> str:
        """Process the latest status update retrieved from an 'Operation-Location' header.
        Raise errors for issues with input document.

        :param azure.core.pipeline.PipelineResponse pipeline_response: The response to extract the status.
        :return: Returns the latest status from the 'Operation-Location' header.
        :rtype: str
        :raises: BadResponse if response has no body, or body does not contain status.
            HttpResponseError if there is an error with the input document.
        """
        response = pipeline_response.http_response
        if _is_empty(response):
            raise BadResponse("The response from long running operation does not contain a body.")

        body = _as_json(response)
        status = body.get("status")
        if not status:
            raise BadResponse("No status found in body")
        if status.lower() == "failed":
            analyze_result = body.get("analyzeResult")
            if analyze_result:
                errors = analyze_result.get("errors")
                if errors:
                    raise_error(response, errors, message="")
        return status


class CopyPolling(OperationResourcePolling):
    """Polling method overrides for copy endpoint."""

    def get_status(self, pipeline_response: PipelineResponse) -> str:
        """Process the latest status update retrieved from an "Operation-Location" header.
        Raise errors for issues occurring during the copy model operation.

        :param azure.core.pipeline.PipelineResponse pipeline_response: The response to extract the status.
        :return: Returns the latest status from the 'Operation-Location' header.
        :rtype: str
        :raises: BadResponse if response has no body, or body does not contain status.
            HttpResponseError if there is an error with the input document.
        """
        response = pipeline_response.http_response
        if _is_empty(response):
            raise BadResponse("The response from long running operation does not contain a body.")

        body = _as_json(response)
        status = body.get("status")
        if not status:
            raise BadResponse("No status found in body")
        if status.lower() == "failed":
            copy_result = body.get("copyResult")
            if copy_result:
                errors = copy_result.get("errors")
                if errors:
                    raise_error(response, errors, message="")
        return status
